--
-- Derivatives.hs
--
-- The vector of time derivatives that compose the system to be integrated.
-- The entries are:
--
--    y[0]  = temperature, in units of 10^9 K
--    y[1]  = the normalized baryon density, M_u * n_b / (T / (10^9 K))^3, units g cm^{-3}
--    y[2]  = the normalized electron chemical potential, Phi / (k * T), dimensionless.
--    y[3]  = neutron abundance
--    y[4]  = proton
--    y[5]  = deuterium
--    y[6]  = tritium
--    y[7]  = 3He
--    y[8]  = 4He
--    y[9]  = 7Li
--    y[10] = 7Be
--


module Derivatives (
        EvolutionFunction,
        -- * Functions that computes the derivatives.
        systemDot,
) where

import Debug.Trace
import Data.Packed.Vector
import Numeric.LinearAlgebra

import Cosmology
import Initialize (nNuclides, t9small)
import Matrices
import Reactions
import Thermal


type EvolutionFunction =  Double         -- ^ time
                       -> Double         -- ^ stepsize
                       -> Vector Double  -- ^ current y(t)
                       -> Vector Double  -- ^ dy / dt

-- | The system of equations.
--
--   Given y(t), compute dy/dt at t.
--
systemDot :: EvolutionFunction
systemDot t h y = let
        t9  = y @> 0
        hv  = y @> 1
        phi = y @> 2
        ys  = subVector 3 nNuclides y
 
        yDots = abundanceDot t h t9 hv ys
        (t9Dot, hvDot, phiDot) = cosmologyDot t t9 hv phi ys yDots

        in
          join [fromList [t9Dot, hvDot, phiDot], yDots]

-- | The rate of change of the nuclide abundances.  Because the rate equations
--   are stiff, this is computed implicitly; i.e., what is computed is the
--   rate of change at the end of an interval h long that would give the abundances
--   specified at the beginning of the interval.
--
abundanceDot :: Double         -- ^ time, units s
             -> Double         -- ^ current stepsize
             -> Double         -- ^ t9
             -> Double         -- ^ hv
             -> Vector Double  -- ^ abundances
             -> Vector Double  -- ^ d(abundances) / dt
abundanceDot t h t9 hv ys = let
        rhob = hv * (t9^3)

        rateMatrix = linearizedReactionMatrix t9 rhob ys
        im         = ident nNuclides

        rh  = scale h rateMatrix
        d   = sub im rh
        ys' = matrixSolve d ys
        d'  = sub ys' ys
        in
          scale (1.0 / h) d'



-- | The function cosmologyDot computes the time derivatives
--   of the cosmic parameters: temperature, normalized density and
--   normalized electron chemical potential.
--
cosmologyDot :: Double                    -- ^ time
             -> Double                    -- ^ t9
             -> Double                    -- ^ hv
             -> Double                    -- ^ phi
             -> Vector Double             -- ^ nuclide abundances
             -> Vector Double             -- ^ d(nuclide abundance) / dt
             -> (Double, Double, Double)  -- ^ d(t9, hv, phi) / dt
cosmologyDot t t9 hv phi ys ydots = let
        hubble = scaleFactorDot t t9 hv phi

        dTemp  = temperatureDot t t9 hv phi hubble ys ydots
        dhv    = densityDot     t t9 hv phi hubble ys ydots
        dPhi   = phiDot         t t9 hv phi hubble ys ydots

        in
          (dTemp, dhv, dPhi)



temperatureDot :: Double         -- ^ time, units s
               -> Double         -- ^ temperature / 10^9 K
               -> Double         -- ^ normalized baryon density, units g cm^{-3}
               -> Double         -- ^ normalized electron chemical potential
               -> Double         -- ^ the hubble parameter
               -> Vector Double  -- ^ vector of abundances
               -> Vector Double  -- ^ vector of abundance time derivatives
               -> Double
temperatureDot t t9 hv phi hubble ys ydots = let
        sumy   =  sumVector ys
        sumzy  =  dot zs  ys
        sumdy  =  sumVector ydots
        summdy =  dot dms ydots
        sumzdy =  dot zs  ydots

        zeta   = 1.38767e-04    -- ^ (3 * k * (10^9 K)) / (2 * c^2 * M_u)

        rhob   = hv * (t9^3)

        drhoEdphi = dphiEpEnergyDensity t9 phi

        dNdphi  = dphiEpDiffNumberDensity  t9 phi
        dNdt9   = dTempEpDiffNumberDensity t9 phi

        dMdt9   =  3.58624e-05 * hv * sumzy * (-3.0 / t9)
        dMdr    = -3.56824e-05 * hv * sumzy
        dMdS    =  3.56824e-05 * hv

        dphidt9 = (1.0 / dNdphi) * (dMdt9 - dNdt9)
        dphidr  = (1.0 / dNdphi) * dMdr
        dphidS  = (1.0 / dNdphi) * dMdS

        drhoPhotdt9 = dTempPhotonEnergyDensity t9           
        drhoEpdt9   = if t9 > t9small
                          then (dTempEpEnergyDensity t9 phi) +
                               (dphiEpEnergyDensity t9 phi) * dphidt9  -- Kawano (D.29)
                          else 0.0
        drhoBdt9    = rhob * zeta * sumy

        electronPressure = if t9 > t9small
                               then epPressure t9 phi
                               else 0.0

        pressure = (photonPressure t9) + electronPressure +
                   rhob * (2.0/ 3.0) * zeta * t9 * sumy  -- Baryon pressure, Kawano (D. 32)
 
        photonDensity   = photonEnergyDensity t9

        electronDensity = if t9 > t9small
                              then epEnergyDensity t9 phi
                              else 0.0

        drhoBaryondtConstT9   = if t9 > t9small
                                    then rhob * (t9 * zeta * sumdy + summdy)  -- Kawano (D. 33)
                                    else 0.0

        drhoElectrondtConstT9 = if t9 > t9small
                                    then drhoEdphi * (dphidr * 3.0 * hubble + dphidS * sumzdy)  -- Kawano (D. 34)
                                    else 0.0

        numer = drhoPhotdt9 + drhoEpdt9 + drhoBdt9
        denom = pressure +
                photonDensity +
                electronDensity +
                (1.0 / (3.0 * hubble)) * (drhoBaryondtConstT9 +
                                          drhoElectrondtConstT9)
        
        in
          -(3.0 * hubble) / (numer / denom)



densityDot :: Double         -- ^ time, units s
           -> Double         -- ^ temperature / 10^9 K
           -> Double         -- ^ normalized density, units g cm^{-3}
           -> Double         -- ^ normalized elctron chemical potential
           -> Double         -- ^ the hubble parameter
           -> Vector Double  -- ^ vector of abundances
           -> Vector Double  -- ^ vector of abundance time derivatives
           -> Double
densityDot t t9 hv phi hubble ys ydots = let
        dt9dt = temperatureDot t t9 hv phi hubble ys ydots
        in
          -3.0 * hv * (hubble + dt9dt / t9)



phiDot :: Double         -- ^ time, units s
       -> Double         -- ^ temperature / 10^9 K
       -> Double         -- ^ normalized density, units g cm^{-3}
       -> Double         -- ^ normalized electron chemical potential, dimensionless
       -> Double         -- ^ hubble parameter
       -> Vector Double  -- ^ vector of abundances
       -> Vector Double  -- ^ vector of abundance time derivatives
       -> Double
phiDot t t9 hv phi hubble ys ydots =
        if t9 < t9small
            then 0.0
            else let
                    sumzy  = dot zs ys

                    dNdphi  = dphiEpDiffNumberDensity  t9 phi
                    dNdt9   = dTempEpDiffNumberDensity t9 phi

                    dMdt9   =  3.58624e-05 * hv * sumzy * (-3.0 / t9)
                    dMdr    = -3.56824e-05 * hv * sumzy
                    dMdS    =  3.56824e-05 * hv

                    dphidt9 = (1.0 / dNdphi) * (dMdt9 - dNdt9)
                    dphidr  = (1.0 / dNdphi) * dMdr
                    dphidS  = (1.0 / dNdphi) * dMdS

                    dt9dt   = temperatureDot t t9 hv phi hubble ys ydots
                    dSdt    = dot zs ydots
                 
                 in
                   (dphidt9 * dt9dt)       +
                   (dphidr * 3.0 * hubble) +
                   (dphidS * dSdt)

