{-# OPTIONS_GHC -fglasgow-exts #-}
--
-- Integrate the rate equations for nucleosynthesis using Heun's method,
-- sometime referred to a second order Runge-Kutta method.
--
-- Gregory Wright, 2 October 2006
--

module Integrate (
        nsynStep,
) where


import Data.Packed.Vector

import Derivatives
import Initialize



-- | The exported interface, nsynStep.  It integrates the rate equations
--   using from time t to t + h.  The timestep is to use for the next step is
--   automatically calculated during the integration.  Initial values of the
--   system are in y.
--
nsynStep :: (Double,            -- ^ starting time
             Double,            -- ^ ending time
             Double,            -- ^ stepsize
             Vector Double)     -- ^ initial values of the system
         -> (Double,            -- ^ new time
             Double,            -- ^ new ending time
             Double,            -- ^ new step size
             Vector Double)     -- ^ new system vector at t + h
nsynStep (t, tlimit, h, ys) = let
        (newYs, newYDots) = rk2Step t h ys systemDot
        h' = chooseStepSize t tlimit h newYs newYDots
        in
          (t + h, tlimit, h', newYs)



rk2Step :: Double              -- ^ time
        -> Double              -- ^ stepsize
        -> Vector Double       -- ^ initial values
        -> EvolutionFunction   -- ^ dy / dt evaluated at t.
        -> (Vector Double,     -- ^ final values
            Vector Double)     -- ^ final d(system) / dt
rk2Step t h ys func = let

        yOneDots = func t h ys
        trialYs  = zipVector (+) ys (mapVector (\e -> e * h) yOneDots)
 
        yTwoDots = func t h trialYs
        finalYs  = zipVector (+) ys (mapVector (\e -> (e * 0.5 * h)) (zipVector (+) yOneDots yTwoDots))

        in
          (finalYs, yTwoDots)



chooseStepSize :: Double         -- ^ time
               -> Double         -- ^ time limit
               -> Double         -- ^ stepsize
               -> Vector Double  -- ^ y
               -> Vector Double  -- ^ dy / dt
               -> Double
chooseStepSize t tlimit h ys yDots = let
        temperature    = ys    @> 0
        temperatureDot = yDots @> 0

        ymin   = 1.0e-25
        ys'    = subVector 3 nNuclides ys
        yDots' = subVector 3 nNuclides yDots

        tempRatio       = abs (temperature / temperatureDot)
        abundanceRatio  = vectorMin (mapVector abs (zipVector (/) ys' yDots'))
        abundanceFactor = vectorMin (mapVector (\yi -> 1.0 + (log yi / log ymin)^2) ys')

        kScale = 1.0
        kTemp = 0.03 * kScale
        kAbundance = 0.3 * kScale
        newStep = min (kAbundance * abundanceFactor * abundanceRatio)
                      (kTemp * tempRatio)

        in
          if (tlimit - t) > newStep
              then if newStep > (1.5 * h)
                      then (1.5 * h)
                      else newStep
              else (tlimit - t)



