--
-- Rates.hs
--
-- Functions to compute the rates and reverse rates of the
-- reactions for that contribute to primordial nucleosynthesis.
--
-- The functions return the rates in units of cm^3 s^{-1} mol^{-1}.
--
--
-- Gregory Wright, 16 June 2006
--
-- Copyright (c) 2006 Antiope Associates LLC, all rights reserved.
--
--
-- References:
--
-- Ajzenberg-Selove, F., Nucl. Phys. A449, 1 (1986).
--
-- Ajzenberg-Selove, F., Nucl. Phys. A490, 1 (1988).
--
-- Ajzenberg-Selove, F., Nucl. Phys. A506, 1 (1990).
--
-- Caughlan G.R. and W.A. Fowler, Atomic Data and Nuclear Data Tables, 40, 283 (1988).
--
-- Eidelman, S., et al. (Particle Data Group), Phys. Lett. B 592, 1 (2004).
-- 
-- Malaney R.A. and W.A. Fowler, Ap. J. 345, L5 (1989).
--
-- Smith, M., L.H. Kawano and R.A. Malaney, Ap. J. Suppl. Ser. 85, 219 (1993).
--
-- Tilley, D.R., H.R. Weller and H.H. Hasan, Nucl. Phys. A474 1 (1987).
--
-- Wagoner, R.V., Ap. J. Suppl. No. 162, 18, 247 (1969).
--



module Rates (
        -- * the type of a reaction
        Reaction,

        -- * functions implementing the reaction rates:

        -- * weak decays:
        np,
        pn,
        tHe3,
        li8He4,
        b12C12,
        c14N14,
        b8He4He4,
        c11B11,
        n12C12,
        n13C13,
        o14N14,
        o15N15,

        -- * Neutron, photon reactions:
        npgd,
        ndgt,
        nHe3gHe4,
        nLi6gLi7,
        nLi7gLi8,
        nB10gB11,
        nB11gB12,
        nC12gC13,
        nC13gC14,
        nN14gN15,

        -- * Neutron, proton reactions:
        nHe3pt,
        nBe7pLi7,
        nC11pB11,
        nN13pC13,
        nN14pC14,
        nO15pN15,

        -- * Neutron, alpha reactions:
        nLi6He4t,
        nBe7He4He4,
        nB10He4Li7,
        nO15He4C12,

        -- * Proton, photon reactions:
        pdgHe3,
        ptgHe4,
        pLi6gBe7,
        pBe7gB8,
        pBe9gB10,
        pB10gC11,
        pB11gC12,
        pC11gN12,
        pC12gN13,
        pC13gN14,
        pC14gN15,
        pN13gO14,
        pN14gO15,
        pN15gO16,

        -- * Proton, neutron reactions:
        pB12nC12,

        -- * Proton, alpha reactions:
        pLi6He4He3,
        pLi7He4He4,
        pBe9He4Li6,
        pB10He4Be7,
        pB12He4Be9,
        pN15He4C12,

        -- * Alpha, photon reactions:
        dHe4gLi6,
        tHe4gLi7,
        aHe3gBe7,
        li6He4gB10,
        li7He4gB11,
        be7He4gC11,
        c12He4gO16,

        -- * Alpha, proton reactions:
        b8He4pC11,
        b10He4pC13,
        b11He4pC14,
        c11He4pN14,
        n12He4pO15,
        n13He4pO16,

        -- * Alpha, neutron reactions:
        li8He4nB11,
        be9He4nC12,
        b10He4nN13,
        b11He4nN14,
        b12He4nN15,
        c13He4nO16,

        -- * Deuteron, neutron and deuteron, proton reactions:
        ddnHe3,
        ddpt,
        tdnHe4,
        dHe3pHe4,
        dBe9nB10,
        dB10pB11,
        dB11nC12,

        -- * Three particle reactions:
        he3He3ppHe4,
        dLi7nHe4He4,
        dBe7pHe4He4,
        nHe4He4gBe9,
        he4He4He4gC12,
        pLi8nHe4He4,
        nB8pHe4He4,
        pBe9dHe4He4,
        pB11He4He4He4,
        nC11He4He4He4,

        -- * Default for unimplemented reactions:
        unimpl
) where

import Initialize


type Reaction = Double -> Double 


neutronLifetime = 885.7        -- neutron lifetime is 885.7 seconds (Eidelman 2004)


-- | Temperature dependent correction to the neutron -> proton decay rate,
--   dimensionless.
--
neutronToProton :: Double -> Double
neutronToProton t9 =
        let
	        zr = t9 / t9electron
        in
	        1.0 + (zr * (  0.565 +
	               zr * ( -6.382 +
	               zr * ( 11.108 +
	               zr * ( 36.492 +
	               zr * ( 27.512))))))


-- | Temperature dependent correction to the proton -> neutron rate, dimensionless.
--
protonToNeutron :: Double -> Double
protonToNeutron t9 =
        let
	        zr = t9 / t9electron
        in
	        exp((-deltaNPmass) / zr) * (zr * (  5.252 +
	                                    zr * (-16.229 +
	                                    zr * ( 18.059 +
	                                    zr * ( 34.181 +
	                                    zr * ( 27.617))))))


--------------------------------------------------------------------------------
-- Weak interaction rates:
--------------------------------------------------------------------------------

-- n -> p

np :: Reaction
np t9 = (neutronToProton t9) / neutronLifetime


-- p -> n

pn :: Reaction
pn t9 = (protonToNeutron t9) / neutronLifetime


-- t -> e- + nu + 3He             (Tilly, Weller and Hasan 1987)

tHe3 :: Reaction
tHe3 t9 = 1.79e-09


-- 8Li -> e- + nu + 4He + 4He     (Ajzenberg-Selove 1988)

li8He4 :: Reaction
li8He4 t9 = 8.27e-01


-- 12B -> e- + nu + 12C           (Ajzenberg-Selove 1990)

b12C12 :: Reaction
b12C12 t9 = 3.43e+01


-- 14C -> e- + nu + 14N           (Ajzenberg-Selove 1986)

c14N14 :: Reaction
c14N14 t9 = 3.834e-12


-- 8B -> e+ + nu + 4He + 4He      (Ajzenberg-Selove 1988)

b8He4He4 :: Reaction
b8He4He4 t9 = 9.00e-01


-- 11C -> e+ + nu + 11B           (Ajzenberg-Selove 1990)

c11B11 :: Reaction
c11B11 t9 = 5.668e-04


-- 12N -> e+ + nu + 12C           (Ajzenberg-Selove 1990)

n12C12 :: Reaction
n12C12 t9 = 6.301e+01


-- 13N -> e+ + nu + 13C           (Ajzenberg-Selove 1986)

n13C13 :: Reaction
n13C13 t9 = 1.159e-03


-- 14O -> e+ + nu + 14N           (Ajzenberg-Selove 1986)

o14N14 :: Reaction
o14N14 t9 = 9.8171e-03


-- 15O -> e+ + nu + 15N           (Ajzenberg-Selove 1986)

o15N15 :: Reaction
o15N15 t9 = 5.6704e-03



--------------------------------------------------------------------------------
-- Neutron and photon reactions
--------------------------------------------------------------------------------

-- p + n -> gamma + d             (Smith, Kawano and Malaney 1992)

npgd :: Reaction
npgd t9 =
        let
            t912  = sqrt t9
        in
            4.742e+04 * (1.0 + t912 * (-0.850    +
                               t912 * ( 0.490    +
                               t912 * (-0.0962   +
                               t912 * ( 8.47e-03 +
                               t912 * (-2.80e-04))))))


-- d + n -> gamma + t             (Wagoner 1969)

ndgt :: Reaction
ndgt t9 = 6.62e+01 * (1.0 + 18.9 * t9)


-- 3He + n -> gamma + 4He         (Wagoner 1969)

nHe3gHe4 :: Reaction
nHe3gHe4 t9 = 6.62e+00 * (1.0 + 905.0 * t9)


-- 6Li + n -> gamma + 7Li         (Malaney and Fowler 1989)

nLi6gLi7 :: Reaction
nLi6gLi7 t9 = 5.10e+03


-- 7Li + n -> gamma + 8Li         (Wagoner 1969)

nLi7gLi8 :: Reaction
nLi7gLi8 t9 =
        let
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            4.90e+03 + 9.96e+03 * t9m32 * exp(-2.62 / t9)


-- 10B + n -> gamma + 11B         (Wagoner 1969)

nB10gB11 :: Reaction
nB10gB11 t9 = 6.62e+04


-- 11B + n -> gamma + 12B         (Malaney and Fowler 1989)

nB11gB12 :: Reaction
nB11gB12 t9 =
        let
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            7.29e+02 + 2.40e+03 * t9m32 * exp(-0.223 / t9)


-- 12C + n -> gamma + 13C         (Wagoner 1969)

nC12gC13 :: Reaction
nC12gC13 t9 = 4.50e+02


-- 13C + n -> gamma + 14C         (Wagoner 1969)

nC13gC14 :: Reaction
nC13gC14 t9 =
        let
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            1.19e+02 + 2.38e+05 * t9m32 * exp(-1.67 / t9)


-- 14N + n -> gamma + 15N         (Wagoner 1969)

nN14gN15 :: Reaction
nN14gN15 t9 = 9.94e+03


--------------------------------------------------------------------------------
-- Neutron and proton reactions
--------------------------------------------------------------------------------

-- 3He + n -> p + t               (Smith, Kawano and Malaney 1992)

nHe3pt :: Reaction
nHe3pt t9 =
        let
            t912 = sqrt t9
        in
            7.21e+08 * (1.0 + t912 * (-0.508 +
                              t912 * ( 0.228)))



-- 7Be + n -> p + 7Li             (Smith, Kawano and Malaney 1992)

nBe7pLi7 :: Reaction
nBe7pLi7 t9 =
        let
            t9m32 = t9 ** (-1.5)
            t913  = t9 ** (1.0 / 3.0)
            t9a'' = t9 / (1.0 + 13.076 * t9)
        in
            9.391e+08 * ((t9a'' / t9) ** 1.5) +
            4.467e+07 * t9m32 * exp(-0.07486 / t9) +
            2.675e+09 * (1.0 + t913 * (-0.560    +
                               t913 * ( 0.179    +
                               t913 * (-0.0283   +
                               t913 * ( 2.21e-03 +
                               t913 * (-6.85e-05))))))


-- 11C + n -> p + 11B             (Caughlan and Fowler 1988)

nC11pB11 :: Reaction
nC11pB11 t9 =
        let
            t912 = sqrt t9
        in
            1.69e+08 * (1.0 - 0.048 * t912 + 0.010 * t9)


-- 13N + n -> p + 13C             (Caughlan and Fowler 1988)

nN13pC13 :: Reaction
nN13pC13 t9 =
       let
           t912 = sqrt t9
       in
           1.88e+08 * (1.0 - 0.167 * t912 + 0.037 * t9)


-- 14N + n -> p + 14C             (Caughlan and Fowler 1988)

nN14pC14 :: Reaction
nN14pC14 t9 =
        let
            t912 = sqrt t9
        in
            2.39e+05 * (1.0 + 0.361 * t912 + 0.502 * t9) +
            1.112e+08 / t912 * exp(-4.983 / t9)


-- 15O + n -> p + 15N             (Caughlan and Fowler 1988)

nO15pN15 :: Reaction
nO15pN15 t9 =
        let
            t912 = sqrt t9
        in
            3.50e+08 * (1.0 + 0.452 * t912 - 0.191 * t9)


--------------------------------------------------------------------------------
-- Neutron, alpha reactions
--------------------------------------------------------------------------------

-- 6Li + n -> 4He + t             (Caughlan and Fowler 1988)

nLi6He4t :: Reaction
nLi6He4t t9 =
        let
            t932  = t9 ** (3.0 / 2.0)
            t9m32 = 1.0 / t932

            t9b   = t9 / (1.0 + 49.18 * t9)
            t9b32 = t9b ** (3.0 / 2.0)
        in
            2.54e+09 * t9m32 * exp(-2.39 / t9) +
            1.68e+08 * (1.0 - 0.261 * t9b32 / t932)


-- 7Be + n -> 4He + 4He           (Wagoner 1969)

nBe7He4He4 :: Reaction
nBe7He4He4 t9 = 2.05e+04 * (1.0 + 3760.0 * t9)


-- 10B + n -> 4He + 7Li           (Caughlan and Fowler 1988)

nB10He4Li7 :: Reaction
nB10He4Li7 t9 = 5.07e+08


-- 15O + n -> 4He + 12C           (Caughlan and Fowler 1988)

nO15He4C12 :: Reaction
nO15He4C12 t9 =
        let
            t912 = sqrt t9
        in
            3.50e+07 * (1.0 + 0.188 * t912 + 0.015 * t9)


--------------------------------------------------------------------------------
-- Proton, photon reactions
--------------------------------------------------------------------------------


-- d + p -> gamma + 3He           (Smith, Kawano and Malaney 1992)

pdgHe3 :: Reaction
pdgHe3 t9 = 
        let
            t9m23 = t9 ** (-2.0 / 3.0)
            t913  = t9 ** ( 1.0 / 3.0)
        in
            2.65e+03 * t9m23 * exp(-3.720 / t913) * (1.0 + t913 * (0.112 +
                                                           t913 * (1.99  +
                                                           t913 * (1.56  +
                                                           t913 * (0.162 +
                                                           t913 * (0.324))))))


-- t + p -> gamma + 4He           (Caughlan and Fowler 1988)

ptgHe4 :: Reaction
ptgHe4 t9 =
        let
            t913  = t9 ** (1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in 
            2.20e+04 * t9m23 * exp(-3.869 / t913) * (1.0 + t913 * (0.108 +
                                                           t913 * (1.68  +
                                                           t913 * (1.26  +
                                                           t913 * (0.551 +
                                                           t913 * (1.06))))))


-- 6Li + p -> gamma + 7Be         (Caughlan and Fowler 1988)

pLi6gBe7 :: Reaction
pLi6gBe7 t9 =
        let
            t9m32 = t9 ** (-3.0 / 2.0)
            t953  = t9 ** (5.0 / 3.0)

            t9c = if t9 > 10.0
                      then 1.0
                      else t9 / (1.0 - 9.69e-02 * t9 + 2.84e-02 * t953 / (1.0 - 9.69e-02 * t9) ** (2.0/3.0))

            t9c56 = t9c ** (5.0 / 6.0)
            t9c13 = t9c ** (1.0 / 3.0)
        in
            6.69e+05 * t9c56 * t9m32 * exp(-8.413 / t9c13)


-- 7Be + p -> gamma + 8B          (Caughlan and Fowler 1988)

pBe7gB8 :: Reaction
pBe7gB8 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            3.11e+05 * t9m23 * exp(-10.262 / t913) +
            2.53e+03 * t9m32 * exp(-7.306 / t9)


-- 9Be + p -> gamma + 10B         (Caughlan and Fowler 1988)

pBe9gB10 :: Reaction
pBe9gB10 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            9.64e+04 * t9m32 * exp(-3.445 / t9) +
            2.72e+06 * t9m32 * exp(-10.620 / t9) +
            1.33e+07 * t9m23 * exp(-10.359 / t913 - (t9 / 0.846)^2) * (1.0 + t913 * (0.040 +
                                                                             t913 * (1.52  +
                                                                             t913 * (0.428 + 
                                                                             t913 * (2.15  +
                                                                             t913 * (1.54))))))

-- 10B + p -> gamma + 11C         (Caughlan and Fowler 1988)

pB10gC11 :: Reaction
pB10gC11 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            1.93e+05 * t9m32 * exp(-12.041 / t9) +
            1.14e+04 * t9m32 * exp(-16.164 / t9) +
            4.61e+05 * t9m23 * exp(-12.062 / t913 - (t9 / 4.402)^2) * (1.0 + t913 * (0.035 +
                                                                             t913 * (0.426 +
                                                                             t913 * (0.103 +
                                                                             t913 * (0.281 +
                                                                             t913 * (0.173))))))


-- 11B + p -> gamma + 12C         (Caughlan and Fowler 1988)

pB11gC12 :: Reaction
pB11gC12 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m15 = t9 ** (-1.0 / 5.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            7.89e+03 * t9m32 * exp(-1.733 / t9) +
            9.68e+04 * t9m15 * exp(-5.617 / t9) +
            4.62e+07 * t9m23 * exp(-12.095 / t913 - (t9 / 0.239)^2) * (1.0 + t913 * (0.035 +
                                                                             t913 * (3.00  +
                                                                             t913 * (0.723 +
                                                                             t913 * (9.91  +
                                                                             t913 * (6.07))))))


-- 11C + p -> gamma + 12N         (Caughlan and Fowler 1988)

pC11gN12 :: Reaction
pC11gN12 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            8.84e+03 * t9m32 * exp(-7.021 / t9) +
            4.24e+04 * t9m23 * exp(-13.658 / t913 - (t9 / 1.627)^2) * (1.0 + t913 * (0.031 +
                                                                             t913 * (3.11  +
                                                                             t913 * (0.665 +
                                                                             t913 * (4.61  +
                                                                             t913 * (2.50))))))


-- 12C + p -> gamma + 13N         (Caughlan and Fowler 1988)

pC12gN13 :: Reaction
pC12gN13 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            1.08e+05 * t9m32 * exp(-4.925 / t9)  +
            2.15e+05 * t9m32 * exp(-18.179 / t9) +
            2.04e+07 * t9m23 * exp(-13.690 / t913 - (t9 / 1.500)^2) * (1.0 + t913 * (0.030 +
                                                                             t913 * (1.19  +
                                                                             t913 * (0.254 +
                                                                             t913 * (2.06  +
                                                                             t913 * (1.12))))))


-- 13C + p -> gamma + 14N         (Caughlan and Fowler 1988)

pC13gN14 :: Reaction
pC13gN14 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m65 = t9 ** (-6.0 / 5.0)
        in
            1.21e+06 * t9m65 * exp(-5.701 / t9) +
            8.01e+07 * t9m23 * exp(-13.717 / t913 - (t9 / 2.000)^2) * (1.0 + t913 * (0.030 +
                                                                             t913 * (0.958 +
                                                                             t913 * (0.204 +
                                                                             t913 * (1.39  +
                                                                             t913 * (0.753))))))


-- 14C + p -> gamma + 15N         (Caughlan and Fowler 1988)

pC14gN15 :: Reaction
pC14gN15 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m13 = 1.0 / t913
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            5.36e+03 * t9m32 * exp(-3.811 / t9) +
            9.82e+04 * t9m13 * exp(-4.739 / t9) +
            6.80e+06 * t9m23 * exp(-13.741 / t913 - (t9 / 5.721)^2) * (1.0 + t913 * (0.030 +
                                                                             t913 * (0.503 +
                                                                             t913 * (0.107 +
                                                                             t913 * (0.213 +
                                                                             t913 * (0.115))))))


-- 13N + p -> gamma + 14O         (Caughlan and Fowler 1988)

pN13gO14 :: Reaction
pN13gO14 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            2.43e+05 * t9m32 * exp(-6.348 / t9) +
            4.04e+07 * t9m23 * exp(-15.202 / t913 - (t9 / 1.191)^2) * (1.0 + t913 * ( 0.027 +
                                                                             t913 * (-0.803 +
                                                                             t913 * (-0.154 +
                                                                             t913 * ( 5.00  +
                                                                             t913 * ( 2.44))))))


-- 14N + p -> gamma + 15O         (Caughlan and Fowler 1988)

pN14gO15 :: Reaction
pN14gO15 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            2.37e+03 * t9m32 * exp(-3.011 / t9) +
            2.19e+04 * exp(-12.530 / t9) +
            4.90e+07 * t9m23 * exp(-15.228 / t913 - (t9 / 3.294)^2) * (1.0 + t913 * ( 0.027 +
                                                                             t913 * (-0.778 +
                                                                             t913 * (-0.149 +
                                                                             t913 * ( 0.261 +
                                                                             t913 * ( 0.127))))))


-- 15N + p -> gamma + 16O         (Caughlan and Fowler 1988)

pN15gO16 :: Reaction
pN15gO16 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            1.11e+04 * t9m32 * exp(-3.328 / t9)  +
            1.49e+04 * t9m32 * exp(-4.665 / t9)  +
            3.80e+06 * t9m32 * exp(-11.048 / t9) +
            9.78e+08 * t9m23 * exp(-15.251 / t913 - (t9 / 0.450)^2) * (1.0 + t913 * (0.027 +
                                                                             t913 * (0.219 +
                                                                             t913 * (0.042 +
                                                                             t913 * (6.83  +
                                                                             t913 * (3.32))))))



--------------------------------------------------------------------------------
-- Proton, neutron reactions
--------------------------------------------------------------------------------


-- 12B + p -> n + 12C             (Wagoner 1969)

pB12nC12 :: Reaction
pB12nC12 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            4.02e+11 * t9m23 * exp(-12.12 / t913)


--------------------------------------------------------------------------------
-- Proton, alpha reactions
--------------------------------------------------------------------------------


-- 6Li + p -> 4He + 3He          (Caughlan and Fowler 1988)

pLi6He4He3 :: Reaction
pLi6He4He3 t9 =
        let
            t9m1  = 1.0 / t9
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            1.33e+10 * t9m32 * exp(-17.763 / t9) +
            1.29e+09 * t9m1  * exp(-21.820 / t9) +
            3.73e+10 * t9m23 * exp(-8.413 / t913 - (t9 / 5.50)^2) * (1.0 + t913 * ( 0.050 +
                                                                           t913 * (-0.061 +
                                                                           t913 * (-0.021 +
                                                                           t913 * ( 0.006 +
                                                                           t913 * ( 0.005))))))


-- 7Li + p -> 4He + 4He           (Smith, Kawano and Malaney 1992)

pLi7He4He4 :: Reaction
pLi7He4He4 t9 =
        let
            t9m23  = t9 ** (-2.0 / 3.0)
            t9m32  = t9 ** (-3.0 / 2.0)
            t913   = t9 ** ( 1.0 / 3.0)
            t9a''' = t9 / (1.0 + 0.759 * t9)
        in
            1.096e+09 * t9m23 * exp(-8.472 / t913) -
            4.830e+08 * t9m32 * (t9a''' ** (5.0 / 6.0)) * exp(-8.472 / (t9a''' ** (1.0 / 3.0))) +
            1.060e+10 * t9m32 * exp(-30.442 / t9) +
            1.550e+06 * t9m32 * exp(-4.478  / t9) +
            1.560e+05 * t9m23 * exp(-8.472 / t913 - ((t9 / 1.696)^2)) * (1.0 + t913 * ( 0.049 +
                                                                               t913 * (-2.498 +
                                                                               t913 * ( 0.860 +
                                                                               t913 * ( 3.518 +
                                                                               t913 * ( 3.08))))))


-- 9Be + p -> 4He + 6Li           (Caughlan and Fowler 1988)

pBe9He4Li6 :: Reaction
pBe9He4Li6 t9 =
        let
            t9m1  = 1.0 / t9
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m34 = t9 ** (-3.0 / 4.0)
        in
            4.51e+08 * t9m1  * exp(-3.046 / t9) +
            6.70e+08 * t9m34 * exp(-5.160 / t9) +
            2.11e+11 * t9m23 * exp(-10.359 / t913 - (t9 / 0.520)^2) * (1.0 + t913 * (0.040 +
                                                                             t913 * (1.09  +
                                                                             t913 * (0.307 +
                                                                             t913 * (3.21  +
                                                                             t913 * (2.30))))))


-- 10B + p -> 4He + 7Be           (Caughlan and Fowler 1988)

pB10He4Be7 :: Reaction
pB10He4Be7 t9 =
        let
            t9m1  = 1.0 / t9
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            2.59e+09 * t9m1  * exp(-12.260 / t9) +
            1.26e+11 * t9m23 * exp(-12.062 / t913 - (t9 / 4.402)^2) * (1.0 + t913 * ( 0.035 +
                                                                             t913 * (-0.498 +
                                                                             t913 * (-0.121 +
                                                                             t913 * ( 0.300 +
                                                                             t913 * ( 0.184))))))


-- 12B + p -> 4He + 9Be           (Wagoner 1969)

pB12He4Be9 :: Reaction
pB12He4Be9 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            2.01e+11 * t9m23 * exp(-12.12 / t913)



-- 15N + p -> 4He + 12C           (Caughlan and Fowler 1988)

pN15He4C12 :: Reaction
pN15He4C12 t9 =
        let
            t912  = sqrt t9
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            1.19e+08 * t9m32 * exp(-3.676 / t9) +
            5.41e+08 / t912  * exp(-8.926 / t9)  +
            4.72e+07 * t9m32 * exp(-7.721 / t9) +
            2.20e+08 * t9m32 * exp(-11.418 / t9) +
            1.08e+12 * t9m23 * exp(-15.251 / t913 - (t9 / 0.522)^2) * (1.0 + t913 * (0.027 +
                                                                             t913 * (2.62  +
                                                                             t913 * (0.501 +
                                                                             t913 * (5.36  +
                                                                             t913 * (2.60))))))


--------------------------------------------------------------------------------
-- Alpha, photon reactions
--------------------------------------------------------------------------------

-- d + 4He -> gamma + 6Li         (Caughlan and Fowler 1988)

dHe4gLi6 :: Reaction
dHe4gLi6 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            8.55e+01 * t9m32 * exp(-8.228 / t9) +
            3.01e+01 * t9m23 * exp(-7.423 / t913) * (1.0 + t913 * (0.056  +
                                                           t913 * (-4.85  +
                                                           t913 * ( 8.85  +
                                                           t913 * (-0.585 +
                                                           t913 * (-0.584))))))


-- t + 4He -> gamma + 7Li         (Smith, Kawano and Malaney 1992)

tHe4gLi7 :: Reaction
tHe4gLi7 t9 =
        let
            t9m23  = t9 ** (-2.0 / 3.0)
            t9m32  = t9 ** (-3.0 / 2.0)
            t913   = t9 ** ( 1.0 / 3.0)
            t9a'   = t9 / (1.0 + 0.1378 * t9)
            t9a13' = t9a' ** (1.0 / 3.0)
        in
            5.109e+05 * (t9a' ** (5.0 / 6.0)) * t9m32 * exp(-8.068 / t9a13') +
            3.032e+05 * t9m23 * exp(-8.090 / t913) * (1.0 + t913 * ( 0.0516   +
                                                            t913 * ( 0.0229   +
                                                            t913 * ( 8.28e-03 +
                                                            t913 * (-3.28e-04 +
                                                            t913 * (-3.01e-04))))))


-- 3He + 4He -> gamma + 7Be       (Smith, Kawano and Malaney 1992)

aHe3gBe7 :: Reaction
aHe3gBe7 t9 =
        let
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
            t913  = t9 ** ( 1.0 / 3.0)
            t912  = sqrt t9
            t9a   = t9 / (1.0 + 0.107 * t9)
            t9a13 = t9a ** (1.0 / 3.0)
        in
            5.938e+06 * (t9a ** (5.0 / 6.0)) * t9m32 * exp(-12.859 / t9a13) +
            4.817e+06 * t9m23 * exp(-14.964 / t913) * (1.0 + t913 * ( 0.0325   +
                                                             t913 * (-1.04e-03 +
                                                             t913 * (-2.37e-04 +
                                                             t913 * (-8.11e-05 +
                                                             t913 * (-4.69e-05))))))


-- 6Li + 4He -> gamma + 10B       (Caughlan and Fowler 1988)

li6He4gB10 :: Reaction
li6He4gB10 t9 =
        let
            t9m1  = 1.0 / t9
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            1.91e+03 * t9m32 * exp(-3.484 / t9) +
            1.01e+04 * t9m1  * exp(-7.269 / t9) +
            4.06e+06 * t9m23 * exp(-18.790 / t913 - (t9 / 1.326)^2) * (1.0 + t913 * (0.022 +
                                                                             t913 * (1.54  +
                                                                             t913 * (0.239 +
                                                                             t913 * (2.20  +
                                                                             t913 * (0.869))))))


-- 7Li + 4He -> gamma + 11B       (Caughlan and Fowler 1988)

li7He4gB11 :: Reaction
li7He4gB11 t9 =
        let
            t9m1  = 1.0 / t9
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            3.33e+02 * t9m32 * exp(-2.977 / t9) +
            4.10e+04 * t9m1  * exp(-6.227 / t9) +
            3.55e+07 * t9m23 * exp(-19.161 / t913 - (t9 / 4.195)^2) * (1.0 + t913 * (0.022 +
                                                                             t913 * (0.775 +
                                                                             t913 * (0.118 +
                                                                             t913 * (0.884 +
                                                                             t913 * (0.342))))))


-- 7Be + 4He -> gamma + 11C       (Caughlan and Fowler 1988)

be7He4gC11 :: Reaction
be7He4gC11 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
            t9m54 = t9 ** (-5.0 / 4.0)
        in
            1.25e+04 * t9m32 * exp(-6.510 / t9) +
            1.29e+05 * t9m54 * exp(-10.039 / t9) +
            8.45e+07 * t9m23 * exp(-23.212 / t913 - (t9 / 4.769)^2) * (1.0 + t913 * (0.018 +
                                                                             t913 * (0.488 +
                                                                             t913 * (0.061 +
                                                                             t913 * (0.296 +
                                                                             t913 * (0.095))))))



-- 12C + 4He -> gamma + 16O       (Caughlan and Fowler 1988)

c12He4gO16 :: Reaction
c12He4gO16 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            1.76e+08 / (t9)^2 / (1.0 + 0.2654 * t9m23)^2 * exp(-32.120 / t913) +
            1.25e+03 * t9m32 * exp(-27.499 / t9) +
            1.43e-02 * (t9)^5 * exp(-15.541 / t9) +
            1.04e+08 / t9^2 * exp(-32.120 / t913 - (t9 / 3.496)^2) /(1.0 + 0.0489 * t9m23)^2



--------------------------------------------------------------------------------
-- Alpha, proton reactions
--------------------------------------------------------------------------------


-- 8B + 4He -> p + 11C            (Wagoner 1969)

b8He4pC11 :: Reaction
b8He4pC11 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            1.08e+15 * t9m23 * exp(-27.36 / t913)


-- 10B + 4He -> p + 13C           (Wagoner 1969)

b10He4pC13 :: Reaction
b10He4pC13 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            9.60e+14 * t9m23 * exp(-27.99 / t913)


-- 11B + 4He -> p + 14C           (Caughlan and Fowler 1988)

b11He4pC14 :: Reaction
b11He4pC14 t9 =
        let
            t938  = t9 ** ( 3.0 / 8.0)
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            5.44e-03 * t9m32 * exp(-2.827 / t9) +
            3.36e+02 * t9m32 * exp(-5.178 / t9) +
            5.32e+06 / t938  * exp(-11.617 / t9) +
            5.37e+11 * t9m23 * exp(-28.234 / t913 - (t9 / 0.347)^2) * (1.0 + t913 * (0.015  +
                                                                             t913 * (5.575  +
                                                                             t913 * (0.576  +
                                                                             t913 * (15.888 +
                                                                             t913 * (4.174))))))


-- 11C + 4He -> p + 14N           (Caughlan and Fowler 1988)

c11He4pN14 :: Reaction
c11He4pN14 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)

            t953  = t9 ** ( 5.0 / 3.0)
            t9a   = t9 / (1.0 + 4.78e-02 * t9 + 7.56e-03 * t953 / (1.0 + 4.78e-02 * t9)**(2.0 / 3.0))  
            t9a13 = t9a ** (1.0 / 3.0)
            t9a56 = t9a ** (5.0 / 6.0)
        in
            7.15e+15 * t9a56 * t9m32 * exp(-31.883 / t9a13)


-- 12N + 4He -> p + 15O           (Caughlan and Fowler 1988)

n12He4pO15 :: Reaction
n12He4pO15 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            5.59e+16 * t9m23 * exp(-35.60 / t913)


-- 13N + 4He -> p + 16O           (Caughlan and Fowler 1988)

n13He4pO16 :: Reaction
n13He4pO16 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)

            t953  = t9 ** (5.0 / 3.0)
            t9b   = t9 /(1.0 + 7.76e-02 * t9 + 2.64e-02 * t953 / (1.0 + 7.76e-02 * t9)**(2.0 / 3.0))
            t9b13 = t9b ** (1.0 / 3.0)
            t9b56 = t9b ** (5.0 / 6.0)
        in
            3.23e+17 * t9b56 * t9m32 * exp(-35.829 / t9b13)


--------------------------------------------------------------------------------
-- Alpha, neutron reactions
--------------------------------------------------------------------------------

-- 8Li + 4He -> n + 11B           (Malaney and Fowler 1989)

li8He4nB11 :: Reaction
li8He4nB11 t9 =
        let
            t9m32 = t9 ** (-3.0 / 2.0)

            t9a   = t9 / (1.0 + t9 / 15.1)
            t9a56 = t9a ** (5.0 / 6.0)
            t9a13 = t9a ** (1.0 / 3.0)
        in
            8.62e+13 * t9a56 * t9m32 *exp(-19.461 / t9a13)


-- 9Be + 4He -> n + 12C           (Caughlan and Fowler 1988)

be9He4nC12 :: Reaction
be9He4nC12 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            7.34e-05 * t9m32 * exp(-1.184 / t9) +
            2.27e-01 * t9m32 * exp(-1.834 / t9) +
            1.26e+05 * t9m32 * exp(-4.179 / t9) +
            2.40e+08 * exp(-12.732 / t9) +
            4.62e+13 * t9m23 * exp(-23.870 / t913 - (t9 / 0.049)^2) * (1.0 + t913 * (0.017 +
                                                                             t913 * (8.57  +
                                                                             t913 * (1.05  +
                                                                             t913 * (74.51 +
                                                                             t913 * (23.15))))))


-- 10B + 4He -> n 13N             (Caughlan and Fowler 1988)

b10He4nN13 :: Reaction
b10He4nN13 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            1.20e+13 * t9m23 * exp(-27.989 / t913 - (t9 / 9.589)^2)


-- 11B + 4He -> n + 14N           (Caughlan and Fowler 1988)

b11He4nN14 :: Reaction
b11He4nN14 t9 =
        let
            t935  = t9 ** ( 3.0 / 5.0)
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            1.79e+00 * t9m32 * exp(-2.827 / t9) +
            1.71e+03 * t9m32 * exp(-5.178 / t9) +
            4.49e+06 * t935  * exp(-8.596 / t9) +
            6.97e+12 * t9m23 * exp(-28.234 / t913 - (t9 / 0.140)^2) * (1.0 + t913 * (0.015  +
                                                                             t913 * (8.115  +
                                                                             t913 * (0.838  +
                                                                             t913 * (39.804 +
                                                                             t913 * (10.456))))))


-- 12B + 4He -> n + 15N           (Wagoner 1969)

b12He4nN15 :: Reaction
b12He4nN15 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            3.04e+15 * t9m23 * exp(-28.45 / t913)


-- 13C+ 4He -> n + 16O            (Caughlan and Fowler 1988)

c13He4nO16 :: Reaction
c13He4nO16 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            3.82e+05 * t9m32 * exp(-9.373 / t9) +
            1.41e+06 * t9m32 * exp(-11.873 / t9) +
            2.00e+09 * t9m32 * exp(-20.409 / t9) +
            2.92e+09 * t9m32 * exp(-29.283 / t9) +
            6.77e+15 * t9m23 * exp(-32.329/t913-(t9/1.284)**2) * (1.0 + t913 * (0.013 +
                                                                        t913 * (2.04  +
                                                                        t913 * (0.184))))


--------------------------------------------------------------------------------
-- Deuteron, neutron and deuteron, proton reactions
--------------------------------------------------------------------------------

-- d + d -> n + 3He               (Smith, Kawano and Malaney 1992)

ddnHe3 :: Reaction
ddnHe3 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            3.95e+08 * t9m23 * exp(-4.259 / t913) * (1.0 + t913 * (0.098    +
                                                           t913 * (0.765    +
                                                           t913 * (0.525    +
                                                           t913 * (9.61e-03 +
                                                           t913 * (0.0167))))))
            

-- d + d -> p + t                 (Smith, Kawano and Malaney 1992)

ddpt :: Reaction
ddpt t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            4.17e+08 * t9m23 * exp(-4.258 / t913) * (1.0 + t913 * (0.098 +
                                                           t913 * (0.518 +
                                                           t913 * (0.355 +
                                                           t913 * (0.010 +
                                                           t913 * (0.018))))))


-- t + d -> n + 4He               (Smith, Kawano and Malaney 1992)

tdnHe4 :: Reaction
tdnHe4 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            8.047e+08 * t9m23 * exp(-0.4857 / t9) +
            1.063e+11 * t9m23 *
            exp(-4.559 / t913 - ((t9 / 0.0754)^2)) * (1.0 + t913 * ( 0.092 +
                                                            t913 * (-0.375 +
                                                            t913 * (-0.242 +
                                                            t913 * (33.82  +
                                                            t913 * (55.42))))))



-- 3He + d -> p + 4He             (Smith, Kawano and Malaney 1992)

dHe3pHe4 :: Reaction
dHe3pHe4 t9 =
        let
            t912  = sqrt t9
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            (5.212e+08 / t912) * exp(-1.762 / t9) +
            5.021e+10 * t9m23 *
            exp(-7.144 / t913 - ((t9 / 0.270)^2)) * (1.0 + t913 * (0.058 +
                                                           t913 * (0.603 +
                                                           t913 * (0.245 +
                                                           t913 * (6.97  +
                                                           t913 * (7.19))))))


-- 9Be + d -> n + 10B             (original Wagoner code)

dBe9nB10 :: Reaction
dBe9nB10 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            7.16e+08 * t9m23 * exp(6.44 - 12.6 / t913)


-- 10B+ d -> p + 11B              (original Wagoner code)

dB10pB11 :: Reaction
dB10pB11 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            9.53e+08 * t9m23 * exp(7.30 - 14.8 / t913)


-- 11B + d -> n + 12C             (original Wagoner code)

dB11nC12 :: Reaction
dB11nC12 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            1.41e+09 * t9m23 * exp(7.40 - 14.8 / t913)


--------------------------------------------------------------------------------
-- Three particle reactions
--------------------------------------------------------------------------------

-- He3 + He3 -> p + p + 4He       (Caughlan and Fowler 1988)

he3He3ppHe4 :: Reaction
he3He3ppHe4 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            6.04e+10 * t9m23 * exp(-12.276 / t913) * (1.0 + t913 * ( 0.034 +
                                                            t913 * (-0.522 +
                                                            t913 * (-0.124 +
                                                            t913 * ( 0.353 +
                                                            t913 * ( 0.213))))))


-- 7Li + d -> n + 4He + 4He       (Caughlan and Fowler 1988)

dLi7nHe4He4 :: Reaction
dLi7nHe4He4 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            2.92e+11 * t9m23 * exp(-10.259 / t913)


-- 7Be + d -> p + 4He + 4He       (Caughlan and Fowler 1988)

dBe7pHe4He4 :: Reaction
dBe7pHe4He4 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
        in
            1.07e+12 * t9m23 * exp(-12.428 / t913)


-- 4He + 4He + n -> gamma + 9Be   (Caughlan and Fowler 1988)

nHe4He4gBe9 :: Reaction
nHe4He4gBe9 t9 = (2.59e-06 / ((1.0 + 0.344 * t9) * t9^2)) * exp(-1.062 / t9)


-- 4He + 4He + 4He -> gamma + 12C (Caughlan and Fowler 1988)

he4He4He4gC12 :: Reaction
he4He4He4gC12 t9 =
        let
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            2.79e-08 * t9m32 * t9m32 * exp(-4.4027 / t9) +
            1.35e-08 * t9m32 * exp(-24.811 / t9)


-- 8Li + p -> n + 4He + 4He       (original Wagoner code)

pLi8nHe4He4 :: Reaction
pLi8nHe4He4 t9 =
        let
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            8.65e+09 * t9m23 * exp(-8.52 / t913 - (t9 / 2.53)^2) +
            2.31e+09 * t9m32 * exp(-4.64 / t9)


-- 8B + n -> p + 4He + 4He        (original Wagoner code)

nB8pHe4He4 :: Reaction
nB8pHe4He4 t9 = 4.02e+08


-- 9Be + p -> d + 4He + 4He       (Caughlan and Fowler 1988)

pBe9dHe4He4 :: Reaction
pBe9dHe4He4 t9 =
        let
            t9m1  = 1.0 / t9
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m34 = t9 ** (-3.0 / 4.0)
        in
            5.79e+08 * t9m1  * exp(-3.046 / t9) +
            8.50e+08 * t9m34 * exp(-5.800 / t9) +
            2.11e+11 * t9m23 * exp(-10.359 / t913 - (t9 / 0.520)^2) * (1.0 + t913 * (0.040 +
                                                                             t913 * (1.09  +
                                                                             t913 * (0.307 +
                                                                             t913 * (3.21  +
                                                                             t913 * (2.30))))))


-- 11B + p -> 4He + 4He + 4He     (Caughlan and Fowler 1988)


pB11He4He4He4 :: Reaction
pB11He4He4He4 t9 =
        let
            t9m1  = 1.0 / t9
            t913  = t9 ** ( 1.0 / 3.0)
            t9m23 = t9 ** (-2.0 / 3.0)
            t9m32 = t9 ** (-3.0 / 2.0)
        in
            4.03e+06 * t9m32 * exp(-1.734 / t9)  +
            6.73e+09 * t9m32 * exp(-6.262 / t9)  +
            3.88e+09 * t9m1  * exp(-14.154 / t9) +
            2.20e+12 * t9m23 * exp(-12.095 / t913 - (t9 / 1.644)^2) * (1.0 + t913 * (0.034 +
                                                                             t913 * (0.140 +
                                                                             t913 * (0.034 +
                                                                             t913 * (0.190 +
                                                                             t913 * (0.116))))))


-- 11C + n -> 4He + 4He + 4He     (Wagoner 1969)

nC11He4He4He4 :: Reaction
nC11He4He4He4 t9 = 1.58e+08



-- | default for unimplemented reactions
--
unimpl :: Reaction
unimpl t9 = 0.0
