{-# OPTIONS_GHC -fglasgow-exts #-}
--
-- The reaction network for nucleosynthesis.
--
-- Gregory Wright, 16 June 2006
--


module Reactions (
        -- * The reaction rates
        linearizedReactionMatrix,
                  
        -- * arrays of the atomic numbers, charges and mass excesses
        ams,
        zs,
        dms
) where


import Control.Monad.ST
import Data.Array
import Data.Packed.Vector
import Data.Packed.Matrix
import Data.Packed.ST
import Maybe

import Initialize
import Matrices
import Rates


type ReactionNumber = Int
type NuclideCode    = Int

-- The number of reactions in the model:
nReactions = 88
reactionNumberList = [ 1 .. nReactions ]



data ReactionType = WeakDecay        --   n1      -> n4    + leptons
                  | NNgN             --   n1 + n2 -> gamma +   n4
                  | NNNN             --   n1 + n2 -> n3    +   n4
                  | WeakFission      --   n1      -> 2 n4  + leptons
                  | NN2N             --   n1 + n2 -> 2 n4
                  | N2NN             -- 2 n1      -> n3    +   n4
                  | N3gN             -- 3 n1      -> gamma +   n4
                  | N2NgN            -- 2 n1 + n2 -> gamma +   n4
                  | NNN2N            --   n1 + n2 -> n3    + 2 n4
                  | NN3N             --   n1 + n2 -> 3 n4
                  | N2N2N            -- 2 n1      -> 2 n3  +   n4


data ReactionInfo = ReactionInfo { reactionNumber :: ReactionNumber    -- Reaction number from Kawano [Ref. 1]
                                 , reactionType   :: ReactionType
                                 , forwardFunc    :: Reaction
                                 , reverseFunc    :: Maybe Reaction
                                 , nuclideA       :: NuclideCode
                                 , nuclideB       :: NuclideCode
                                 , nuclideC       :: NuclideCode
                                 , nuclideD       :: NuclideCode
                                 , revCoeff       :: Double
                                 , qValue         :: Double
                                 , reactionName   :: String
                                 } 


-- | Function giving the nuclide numbers for a reaction type.
--
--   This function returns the number of nuclides involved in the reaction
--
--           N_i(^{A_i}{Z_i}) + N_j(^{A_j}{Z_j}) <-> N_k(^{A_k}{Z_k}) + N_l(^{A_l}{Z_l})
--
reactionConfig :: ReactionType -> (Int, Int, Int, Int)
reactionConfig t =
        case t of
                WeakDecay   -> (1, 0, 0, 1)
                NNgN        -> (1, 1, 0, 1)
                NNNN        -> (1, 1, 1, 1)
                WeakFission -> (1, 0, 0, 2)
                NN2N        -> (1, 1, 0, 2)
                N2NN        -> (2, 0, 1, 1)
                N3gN        -> (3, 0, 1, 1)
                N2NgN       -> (2, 1, 0, 1)
                NNN2N       -> (1, 1, 1, 2)
                NN3N        -> (1, 1, 0, 3)
                N2N2N       -> (2, 0, 2, 1)



-- | Table of reaction information.
--
--   The data are taken from Kawano's nucleosynthesis code [Ref. 1].
--
--                         reac#                       type                     function                  rev-func             n1             n2           n3             n4            rev-coeff            q-value                 name
reactionTable :: Array Int ReactionInfo
reactionTable = listArray (1, nReactions) [
        ReactionInfo { reactionNumber =  1, reactionType = WeakDecay,   forwardFunc = np,            reverseFunc = Just pn, nuclideA =  1, nuclideB =  0, nuclideC =  0, nuclideD =  2, revCoeff =  0.0  , qValue =   0.0  , reactionName = "n -> p"         },        
        ReactionInfo { reactionNumber =  2, reactionType = WeakDecay,   forwardFunc = tHe3,          reverseFunc = Nothing, nuclideA =  4, nuclideB =  0, nuclideC =  0, nuclideD =  5, revCoeff =  0.0  , qValue =   0.0  , reactionName = "H3 -> He3"      },
        ReactionInfo { reactionNumber =  3, reactionType = WeakFission, forwardFunc = li8He4,        reverseFunc = Nothing, nuclideA = 10, nuclideB =  0, nuclideC =  0, nuclideD =  6, revCoeff =  0.0  , qValue =   0.0  , reactionName = "Li8 -> 2He4"    },
        ReactionInfo { reactionNumber =  4, reactionType = WeakDecay,   forwardFunc = b12C12,        reverseFunc = Nothing, nuclideA = 16, nuclideB =  0, nuclideC =  0, nuclideD = 17, revCoeff =  0.0  , qValue =   0.0  , reactionName = "B12 -> C12"     },
        ReactionInfo { reactionNumber =  5, reactionType = WeakDecay,   forwardFunc = c14N14,        reverseFunc = Nothing, nuclideA = 21, nuclideB =  0, nuclideC =  0, nuclideD = 22, revCoeff =  0.0  , qValue =   0.0  , reactionName = "C14 -> N14"     },
        ReactionInfo { reactionNumber =  6, reactionType = WeakFission, forwardFunc = b8He4He4,      reverseFunc = Nothing, nuclideA = 11, nuclideB =  0, nuclideC =  0, nuclideD =  6, revCoeff =  0.0  , qValue =   0.0  , reactionName = "B8 -> 2He4"     },
        ReactionInfo { reactionNumber =  7, reactionType = WeakDecay,   forwardFunc = c11B11,        reverseFunc = Nothing, nuclideA = 15, nuclideB =  0, nuclideC =  0, nuclideD = 14, revCoeff =  0.0  , qValue =   0.0  , reactionName = "C11 -> B11"     },
        ReactionInfo { reactionNumber =  8, reactionType = WeakDecay,   forwardFunc = n12C12,        reverseFunc = Nothing, nuclideA = 18, nuclideB =  0, nuclideC =  0, nuclideD = 17, revCoeff =  0.0  , qValue =   0.0  , reactionName = "N12 -> C12"     },
        ReactionInfo { reactionNumber =  9, reactionType = WeakDecay,   forwardFunc = n13C13,        reverseFunc = Nothing, nuclideA = 20, nuclideB =  0, nuclideC =  0, nuclideD = 19, revCoeff =  0.0  , qValue =   0.0  , reactionName = "N13 -> C13"     },
        ReactionInfo { reactionNumber = 10, reactionType = WeakDecay,   forwardFunc = o14N14,        reverseFunc = Nothing, nuclideA = 23, nuclideB =  0, nuclideC =  0, nuclideD = 22, revCoeff =  0.0  , qValue =   0.0  , reactionName = "O14 -> N14"     },
        ReactionInfo { reactionNumber = 11, reactionType = WeakDecay,   forwardFunc = o15N15,        reverseFunc = Nothing, nuclideA = 25, nuclideB =  0, nuclideC =  0, nuclideD = 24, revCoeff =  0.0  , qValue =   0.0  , reactionName = "O15 -> N15"     },
        ReactionInfo { reactionNumber = 12, reactionType = NNgN,        forwardFunc = npgd,          reverseFunc = Nothing, nuclideA =  2, nuclideB =  1, nuclideC =  0, nuclideD =  3, revCoeff =  0.471, qValue =  25.82 , reactionName = "H(n, g)H2"      },
        ReactionInfo { reactionNumber = 13, reactionType = NNgN,        forwardFunc = ndgt,          reverseFunc = Nothing, nuclideA =  3, nuclideB =  1, nuclideC =  0, nuclideD =  4, revCoeff =  1.63 , qValue =  72.62 , reactionName = "H2(n, g)H3"     },
        ReactionInfo { reactionNumber = 14, reactionType = NNgN,        forwardFunc = nHe3gHe4,      reverseFunc = Nothing, nuclideA =  5, nuclideB =  1, nuclideC =  0, nuclideD =  6, revCoeff =  2.61 , qValue = 238.81 , reactionName = "He3(n, g)He4"   },
        ReactionInfo { reactionNumber = 15, reactionType = NNgN,        forwardFunc = nLi6gLi7,      reverseFunc = Nothing, nuclideA =  7, nuclideB =  1, nuclideC =  0, nuclideD =  8, revCoeff =  1.19 , qValue =  84.17 , reactionName = "Li6(n, g)Li7"   },
        ReactionInfo { reactionNumber = 16, reactionType = NNNN,        forwardFunc = nHe3pt,        reverseFunc = Nothing, nuclideA =  5, nuclideB =  1, nuclideC =  2, nuclideD =  4, revCoeff =  1.002, qValue =   8.863, reactionName = "He3(n, p)H3"    },
        ReactionInfo { reactionNumber = 17, reactionType = NNNN,        forwardFunc = nBe7pLi7,      reverseFunc = Nothing, nuclideA =  9, nuclideB =  1, nuclideC =  2, nuclideD =  8, revCoeff =  0.998, qValue =  19.081, reactionName = "Be7(n, p)Li7"   },
        ReactionInfo { reactionNumber = 18, reactionType = NNNN,        forwardFunc = nLi6He4t,      reverseFunc = Nothing, nuclideA =  7, nuclideB =  1, nuclideC =  4, nuclideD =  6, revCoeff =  1.070, qValue =  55.494, reactionName = "Li6(n, a)H3"    },
        ReactionInfo { reactionNumber = 19, reactionType = NN2N,        forwardFunc = nBe7He4He4,    reverseFunc = Nothing, nuclideA =  9, nuclideB =  1, nuclideC =  0, nuclideD =  6, revCoeff =  4.70 , qValue = 220.39 , reactionName = "Be7(n, a)He4"   },
        ReactionInfo { reactionNumber = 20, reactionType = NNgN,        forwardFunc = pdgHe3,        reverseFunc = Nothing, nuclideA =  3, nuclideB =  2, nuclideC =  0, nuclideD =  5, revCoeff =  1.63 , qValue =  63.750, reactionName = "H2(p, g)He3"    },
        ReactionInfo { reactionNumber = 21, reactionType = NNgN,        forwardFunc = ptgHe4,        reverseFunc = Nothing, nuclideA =  4, nuclideB =  2, nuclideC =  0, nuclideD =  6, revCoeff =  2.61 , qValue = 229.932, reactionName = "H3(p, g)He4"    },
        ReactionInfo { reactionNumber = 22, reactionType = NNgN,        forwardFunc = pLi6gBe7,      reverseFunc = Nothing, nuclideA =  7, nuclideB =  2, nuclideC =  0, nuclideD =  9, revCoeff =  1.19 , qValue =  65.054, reactionName = "Li6(p, g)Be7"   },
        ReactionInfo { reactionNumber = 23, reactionType = NNNN,        forwardFunc = pLi6He4He3,    reverseFunc = Nothing, nuclideA =  7, nuclideB =  2, nuclideC =  5, nuclideD =  6, revCoeff =  1.07 , qValue =  46.631, reactionName = "Li6(p, a)He3"   },
        ReactionInfo { reactionNumber = 24, reactionType = NN2N,        forwardFunc = pLi7He4He4,    reverseFunc = Nothing, nuclideA =  8, nuclideB =  2, nuclideC =  0, nuclideD =  6, revCoeff =  4.69 , qValue = 201.291, reactionName = "Li7(p, a)He4"   },
        ReactionInfo { reactionNumber = 25, reactionType = NNgN,        forwardFunc = dHe4gLi6,      reverseFunc = Nothing, nuclideA =  6, nuclideB =  3, nuclideC =  0, nuclideD =  7, revCoeff =  1.53 , qValue =  17.118, reactionName = "H2(a, g)Li6"    },
        ReactionInfo { reactionNumber = 26, reactionType = NNgN,        forwardFunc = tHe4gLi7,      reverseFunc = Nothing, nuclideA =  6, nuclideB =  4, nuclideC =  0, nuclideD =  8, revCoeff =  1.11 , qValue =  28.640, reactionName = "H3(a, g)Li7"    },
        ReactionInfo { reactionNumber = 27, reactionType = NNgN,        forwardFunc = aHe3gBe7,      reverseFunc = Nothing, nuclideA =  6, nuclideB =  5, nuclideC =  0, nuclideD =  9, revCoeff =  1.11 , qValue =  18.423, reactionName = "He3(a, g)Be7"   },
        ReactionInfo { reactionNumber = 28, reactionType = N2NN,        forwardFunc = ddnHe3,        reverseFunc = Nothing, nuclideA =  3, nuclideB =  0, nuclideC =  1, nuclideD =  5, revCoeff =  1.73 , qValue =  37.935, reactionName = "H2(d, n)He3"    },
        ReactionInfo { reactionNumber = 29, reactionType = N2NN,        forwardFunc = ddpt,          reverseFunc = Nothing, nuclideA =  3, nuclideB =  0, nuclideC =  2, nuclideD =  4, revCoeff =  1.73 , qValue =  46.798, reactionName = "H2(d, p)H3"     },
        ReactionInfo { reactionNumber = 30, reactionType = NNNN,        forwardFunc = tdnHe4,        reverseFunc = Nothing, nuclideA =  4, nuclideB =  3, nuclideC =  1, nuclideD =  6, revCoeff =  5.54 , qValue = 204.117, reactionName = "H3(d, n)He4"    },
        ReactionInfo { reactionNumber = 31, reactionType = NNNN,        forwardFunc = dHe3pHe4,      reverseFunc = Nothing, nuclideA =  5, nuclideB =  3, nuclideC =  2, nuclideD =  6, revCoeff =  5.55 , qValue = 212.980, reactionName = "He3(d, p)He4"   },
        ReactionInfo { reactionNumber = 32, reactionType = N2N2N,       forwardFunc = he3He3ppHe4,   reverseFunc = Nothing, nuclideA =  5, nuclideB =  0, nuclideC =  2, nuclideD =  6, revCoeff =  3.39 , qValue = 149.230, reactionName = "He3(He3,2p)He4" },
        ReactionInfo { reactionNumber = 33, reactionType = NNN2N,       forwardFunc = dLi7nHe4He4,   reverseFunc = Nothing, nuclideA =  8, nuclideB =  3, nuclideC =  1, nuclideD =  6, revCoeff =  9.95 , qValue = 175.476, reactionName = "Li7(d,na)He4"   },
        ReactionInfo { reactionNumber = 34, reactionType = NNN2N,       forwardFunc = dBe7pHe4He4,   reverseFunc = Nothing, nuclideA =  9, nuclideB =  3, nuclideC =  2, nuclideD =  6, revCoeff =  9.97 , qValue = 194.557, reactionName = "Be7(d,pa)He4"   },
        ReactionInfo { reactionNumber = 35, reactionType = NNgN,        forwardFunc = nLi7gLi8,      reverseFunc = Nothing, nuclideA =  8, nuclideB =  1, nuclideC =  0, nuclideD = 10, revCoeff =  1.31 , qValue =  23.59 , reactionName = "Li7(n,g)Li8"    },
        ReactionInfo { reactionNumber = 36, reactionType = NNgN,        forwardFunc = nB10gB11,      reverseFunc = Nothing, nuclideA = 13, nuclideB =  1, nuclideC =  0, nuclideD = 14, revCoeff =  3.04 , qValue = 132.95 , reactionName = "B10(n,g)B11"    },
        ReactionInfo { reactionNumber = 37, reactionType = NNgN,        forwardFunc = nB11gB12,      reverseFunc = Nothing, nuclideA = 14, nuclideB =  1, nuclideC =  0, nuclideD = 16, revCoeff =  2.34 , qValue =  39.10 , reactionName = "B11(n,g)B12"    },
        ReactionInfo { reactionNumber = 38, reactionType = NNNN,        forwardFunc = nC11pB11,      reverseFunc = Nothing, nuclideA = 15, nuclideB =  1, nuclideC =  2, nuclideD = 14, revCoeff =  1.002, qValue =  32.080, reactionName = "C11(n,p)B11"    },
        ReactionInfo { reactionNumber = 39, reactionType = NNNN,        forwardFunc = nB10He4Li7,    reverseFunc = Nothing, nuclideA = 13, nuclideB =  1, nuclideC =  6, nuclideD =  8, revCoeff =  0.758, qValue =  32.382, reactionName = "B10(n,a)Li7"    },
        ReactionInfo { reactionNumber = 40, reactionType = NNgN,        forwardFunc = pBe7gB8,       reverseFunc = Nothing, nuclideA =  9, nuclideB =  2, nuclideC =  0, nuclideD = 11, revCoeff =  1.30 , qValue =   1.595, reactionName = "Be7(p,g)B8"     },
        ReactionInfo { reactionNumber = 41, reactionType = NNgN,        forwardFunc = pBe9gB10,      reverseFunc = Nothing, nuclideA = 12, nuclideB =  2, nuclideC =  0, nuclideD = 13, revCoeff =  0.973, qValue =  76.427, reactionName = "Be9(p,g)B10"    },
        ReactionInfo { reactionNumber = 42, reactionType = NNgN,        forwardFunc = pB10gC11,      reverseFunc = Nothing, nuclideA = 13, nuclideB =  2, nuclideC =  0, nuclideD = 15, revCoeff =  3.03 , qValue = 100.840, reactionName = "B10(p,g)C11"    },
        ReactionInfo { reactionNumber = 43, reactionType = NNgN,        forwardFunc = pB11gC12,      reverseFunc = Nothing, nuclideA = 14, nuclideB =  2, nuclideC =  0, nuclideD = 17, revCoeff =  7.01 , qValue = 185.173, reactionName = "B11(p,g)C12"    },
        ReactionInfo { reactionNumber = 44, reactionType = NNgN,        forwardFunc = pC11gN12,      reverseFunc = Nothing, nuclideA = 15, nuclideB =  2, nuclideC =  0, nuclideD = 18, revCoeff =  2.33 , qValue =   6.975, reactionName = "C11(p,g)N12"    },
        ReactionInfo { reactionNumber = 45, reactionType = NNNN,        forwardFunc = pB12nC12,      reverseFunc = Nothing, nuclideA = 16, nuclideB =  2, nuclideC =  1, nuclideD = 17, revCoeff =  3.00 , qValue = 146.08 , reactionName = "B12(p,n)C12"    },
        ReactionInfo { reactionNumber = 46, reactionType = NNNN,        forwardFunc = pBe9He4Li6,    reverseFunc = Nothing, nuclideA = 12, nuclideB =  2, nuclideC =  6, nuclideD =  7, revCoeff =  0.618, qValue =  24.674, reactionName = "Be9(p,a)Li6"    },
        ReactionInfo { reactionNumber = 47, reactionType = NNNN,        forwardFunc = pB10He4Be7,    reverseFunc = Nothing, nuclideA = 13, nuclideB =  2, nuclideC =  6, nuclideD =  9, revCoeff =  0.754, qValue =  13.301, reactionName = "B10(p,a)Be7"    },
        ReactionInfo { reactionNumber = 48, reactionType = NNNN,        forwardFunc = pB12He4Be9,    reverseFunc = Nothing, nuclideA = 16, nuclideB =  2, nuclideC =  6, nuclideD = 12, revCoeff =  0.292, qValue =  79.89 , reactionName = "B12(p,a)Be9"    },
        ReactionInfo { reactionNumber = 49, reactionType = NNgN,        forwardFunc = li6He4gB10,    reverseFunc = Nothing, nuclideA =  7, nuclideB =  6, nuclideC =  0, nuclideD = 13, revCoeff =  1.58 , qValue =  51.753, reactionName = "Li6(a,g)B10"    },
        ReactionInfo { reactionNumber = 50, reactionType = NNgN,        forwardFunc = li7He4gB11,    reverseFunc = Nothing, nuclideA =  8, nuclideB =  6, nuclideC =  0, nuclideD = 14, revCoeff =  4.02 , qValue = 100.538, reactionName = "Li7(a,g)B11"    },
        ReactionInfo { reactionNumber = 51, reactionType = NNgN,        forwardFunc = be7He4gC11,    reverseFunc = Nothing, nuclideA =  9, nuclideB =  6, nuclideC =  0, nuclideD = 15, revCoeff =  4.02 , qValue =  87.539, reactionName = "Be7(a,g)C11"    },
        ReactionInfo { reactionNumber = 52, reactionType = NNNN,        forwardFunc = b8He4pC11,     reverseFunc = Nothing, nuclideA = 11, nuclideB =  6, nuclideC =  2, nuclideD = 15, revCoeff =  3.08 , qValue =  86.00 , reactionName = "B8(a,p)C11"     },
        ReactionInfo { reactionNumber = 53, reactionType = NNNN,        forwardFunc = li8He4nB11,    reverseFunc = Nothing, nuclideA = 10, nuclideB =  6, nuclideC =  1, nuclideD = 14, revCoeff =  3.07 , qValue =  76.96 , reactionName = "Li8(a,n)B11"    }, 
        ReactionInfo { reactionNumber = 54, reactionType = NNNN,        forwardFunc = be9He4nC12,    reverseFunc = Nothing, nuclideA = 12, nuclideB =  6, nuclideC =  1, nuclideD = 17, revCoeff = 10.3  , qValue =  66.160, reactionName = "Be9(a,n)C12"    },
        ReactionInfo { reactionNumber = 55, reactionType = NNNN,        forwardFunc = dBe9nB10,      reverseFunc = Nothing, nuclideA = 12, nuclideB =  3, nuclideC =  1, nuclideD = 13, revCoeff =  2.07 , qValue =  50.63 , reactionName = "Be9(d,n)B10"    },
        ReactionInfo { reactionNumber = 56, reactionType = NNNN,        forwardFunc = dB10pB11,      reverseFunc = Nothing, nuclideA = 13, nuclideB =  3, nuclideC =  2, nuclideD = 14, revCoeff =  6.44 , qValue = 107.13 , reactionName = "B10(d,p)B11"    },
        ReactionInfo { reactionNumber = 57, reactionType = NNNN,        forwardFunc = dB11nC12,      reverseFunc = Nothing, nuclideA = 14, nuclideB =  3, nuclideC =  1, nuclideD = 17, revCoeff = 14.9  , qValue = 159.36 , reactionName = "B11(d,n)C12"    },
        ReactionInfo { reactionNumber = 58, reactionType = N2NgN,       forwardFunc = nHe4He4gBe9,   reverseFunc = Nothing, nuclideA =  6, nuclideB =  1, nuclideC =  0, nuclideD = 12, revCoeff =  0.584, qValue =  18.260, reactionName = "He4(an,g)Be9"   },
        ReactionInfo { reactionNumber = 59, reactionType = N3gN,        forwardFunc = he4He4He4gC12, reverseFunc = Nothing, nuclideA =  6, nuclideB =  0, nuclideC =  0, nuclideD = 17, revCoeff =  2.00 , qValue =  84.420, reactionName = "He4(2a,g)C12"   },
        ReactionInfo { reactionNumber = 60, reactionType = NNN2N,       forwardFunc = pLi8nHe4He4,   reverseFunc = Nothing, nuclideA = 10, nuclideB =  2, nuclideC =  1, nuclideD =  6, revCoeff =  3.58 , qValue = 177.73 , reactionName = "Li8(p,na)He4"   },
        ReactionInfo { reactionNumber = 61, reactionType = NNN2N,       forwardFunc = nB8pHe4He4,    reverseFunc = Nothing, nuclideA = 11, nuclideB =  1, nuclideC =  2, nuclideD =  6, revCoeff =  3.58 , qValue = 218.82 , reactionName = "B8(n,pa)He4"    },
        ReactionInfo { reactionNumber = 62, reactionType = NNN2N,       forwardFunc = pBe9dHe4He4,   reverseFunc = Nothing, nuclideA = 12, nuclideB =  2, nuclideC =  3, nuclideD =  6, revCoeff =  0.807, qValue =   7.555, reactionName = "Be9(p,da)He4"   },
        ReactionInfo { reactionNumber = 63, reactionType = NN3N,        forwardFunc = pB11He4He4He4, reverseFunc = Nothing, nuclideA = 14, nuclideB =  2, nuclideC =  0, nuclideD =  6, revCoeff =  3.50 , qValue = 100.753, reactionName = "B11(p,2a)He4"   },
        ReactionInfo { reactionNumber = 64, reactionType = NN3N,        forwardFunc = nC11He4He4He4, reverseFunc = Nothing, nuclideA = 15, nuclideB =  1, nuclideC =  0, nuclideD =  6, revCoeff =  3.49 , qValue = 132.83 , reactionName = "C11(n,2a)He4"   },
        ReactionInfo { reactionNumber = 65, reactionType = NNgN,        forwardFunc = nC12gC13,      reverseFunc = Nothing, nuclideA = 17, nuclideB =  1, nuclideC =  0, nuclideD = 19, revCoeff =  0.886, qValue =  57.41 , reactionName = "C12(n,g)C13"    },
        ReactionInfo { reactionNumber = 66, reactionType = NNgN,        forwardFunc = nC13gC14,      reverseFunc = Nothing, nuclideA = 19, nuclideB =  1, nuclideC =  0, nuclideD = 21, revCoeff =  3.58 , qValue =  94.88 , reactionName = "C13(n,g)C14"    },
        ReactionInfo { reactionNumber = 67, reactionType = NNgN,        forwardFunc = nN14gN15,      reverseFunc = Nothing, nuclideA = 22, nuclideB =  1, nuclideC =  0, nuclideD = 24, revCoeff =  2.71 , qValue = 125.74 , reactionName = "N14(n,g)N15"    },
        ReactionInfo { reactionNumber = 68, reactionType = NNNN,        forwardFunc = nN13pC13,      reverseFunc = Nothing, nuclideA = 20, nuclideB =  1, nuclideC =  2, nuclideD = 19, revCoeff =  1.002, qValue =  34.846, reactionName = "N13(n,p)C13"    },
        ReactionInfo { reactionNumber = 69, reactionType = NNNN,        forwardFunc = nN14pC14,      reverseFunc = Nothing, nuclideA = 22, nuclideB =  1, nuclideC =  2, nuclideD = 21, revCoeff =  3.003, qValue =   7.263, reactionName = "N14(n,p)C14"    },
        ReactionInfo { reactionNumber = 70, reactionType = NNNN,        forwardFunc = nO15pN15,      reverseFunc = Nothing, nuclideA = 25, nuclideB =  1, nuclideC =  2, nuclideD = 24, revCoeff =  1.002, qValue =  41.037, reactionName = "O15(n,p)N15"    },
        ReactionInfo { reactionNumber = 71, reactionType = NNNN,        forwardFunc = nO15He4C12,    reverseFunc = Nothing, nuclideA = 25, nuclideB =  1, nuclideC =  6, nuclideD = 17, revCoeff =  0.709, qValue =  98.661, reactionName = "O15(n,a)C12"    },
        ReactionInfo { reactionNumber = 72, reactionType = NNgN,        forwardFunc = pC12gN13,      reverseFunc = Nothing, nuclideA = 17, nuclideB =  2, nuclideC =  0, nuclideD = 20, revCoeff =  0.884, qValue =  22.553, reactionName = "C12(p,g)N13"    },
        ReactionInfo { reactionNumber = 73, reactionType = NNgN,        forwardFunc = pC13gN14,      reverseFunc = Nothing, nuclideA = 19, nuclideB =  2, nuclideC =  0, nuclideD = 22, revCoeff =  1.19 , qValue =  87.621, reactionName = "C13(p,g)N14"    },
        ReactionInfo { reactionNumber = 74, reactionType = NNgN,        forwardFunc = pC14gN15,      reverseFunc = Nothing, nuclideA = 21, nuclideB =  2, nuclideC =  0, nuclideD = 24, revCoeff =  0.900, qValue = 118.452, reactionName = "C14(p,g)N15"    },
        ReactionInfo { reactionNumber = 75, reactionType = NNgN,        forwardFunc = pN13gO14,      reverseFunc = Nothing, nuclideA = 20, nuclideB =  2, nuclideC =  0, nuclideD = 23, revCoeff =  3.57 , qValue =  53.706, reactionName = "N13(p,g)O14"    },
        ReactionInfo { reactionNumber = 76, reactionType = NNgN,        forwardFunc = pN14gO15,      reverseFunc = Nothing, nuclideA = 22, nuclideB =  2, nuclideC =  0, nuclideD = 25, revCoeff =  2.70 , qValue =  84.678, reactionName = "N14(p,g)O15"    },
        ReactionInfo { reactionNumber = 77, reactionType = NNgN,        forwardFunc = pN15gO16,      reverseFunc = Nothing, nuclideA = 24, nuclideB =  2, nuclideC =  0, nuclideD = 26, revCoeff =  3.62 , qValue = 140.734, reactionName = "N15(p,g)O16"    },
        ReactionInfo { reactionNumber = 78, reactionType = NNNN,        forwardFunc = pN15He4C12,    reverseFunc = Nothing, nuclideA = 24, nuclideB =  2, nuclideC =  6, nuclideD = 17, revCoeff =  0.706, qValue =  57.623, reactionName = "N15(p,a)C12"    },
        ReactionInfo { reactionNumber = 79, reactionType = NNgN,        forwardFunc = c12He4gO16,    reverseFunc = Nothing, nuclideA = 17, nuclideB =  6, nuclideC =  0, nuclideD = 26, revCoeff =  5.13 , qValue =  83.111, reactionName = "C12(a,g)O16"    },
        ReactionInfo { reactionNumber = 80, reactionType = NNNN,        forwardFunc = b10He4pC13,    reverseFunc = Nothing, nuclideA = 13, nuclideB =  6, nuclideC =  2, nuclideD = 19, revCoeff =  9.36 , qValue =  47.16 , reactionName = "B10(a,p)C13"    },
        ReactionInfo { reactionNumber = 81, reactionType = NNNN,        forwardFunc = b11He4pC14,    reverseFunc = Nothing, nuclideA = 14, nuclideB =  6, nuclideC =  2, nuclideD = 21, revCoeff = 11.0  , qValue =   9.098, reactionName = "B11(a,p)C14"    },
        ReactionInfo { reactionNumber = 82, reactionType = NNNN,        forwardFunc = c11He4pN14,    reverseFunc = Nothing, nuclideA = 15, nuclideB =  6, nuclideC =  2, nuclideD = 22, revCoeff =  3.68 , qValue =  33.915, reactionName = "C11(a,p)N14"    },
        ReactionInfo { reactionNumber = 83, reactionType = NNNN,        forwardFunc = n12He4pO15,    reverseFunc = Nothing, nuclideA = 18, nuclideB =  6, nuclideC =  2, nuclideD = 25, revCoeff =  4.26 , qValue = 111.87 , reactionName = "N12(a,p)O15"    },
        ReactionInfo { reactionNumber = 84, reactionType = NNNN,        forwardFunc = n13He4pO16,    reverseFunc = Nothing, nuclideA = 20, nuclideB =  6, nuclideC =  2, nuclideD = 26, revCoeff =  5.81 , qValue =  60.557, reactionName = "N13(a,p)O16"    },
        ReactionInfo { reactionNumber = 85, reactionType = NNNN,        forwardFunc = b10He4nN13,    reverseFunc = Nothing, nuclideA = 13, nuclideB =  6, nuclideC =  1, nuclideD = 20, revCoeff =  9.34 , qValue =  12.287, reactionName = "B10(a,n)N13"    },
        ReactionInfo { reactionNumber = 86, reactionType = NNNN,        forwardFunc = b11He4nN14,    reverseFunc = Nothing, nuclideA = 14, nuclideB =  6, nuclideC =  1, nuclideD = 22, revCoeff =  3.67 , qValue =   1.835, reactionName = "B11(a,n)N14"    },
        ReactionInfo { reactionNumber = 87, reactionType = NNNN,        forwardFunc = b12He4nN15,    reverseFunc = Nothing, nuclideA = 16, nuclideB =  6, nuclideC =  1, nuclideD = 24, revCoeff =  4.25 , qValue =  88.47 , reactionName = "B12(a,n)N15"    },
        ReactionInfo { reactionNumber = 88, reactionType = NNNN,        forwardFunc = c13He4nO16,    reverseFunc = Nothing, nuclideA = 19, nuclideB =  6, nuclideC =  1, nuclideD = 26, revCoeff =  5.79 , qValue =  25.711, reactionName = "C13(a,n)O16"    } ]



data NuclideInfo = NuclideInfo { atomicNumber :: Int
                               , charge       :: Int
                               , massExcess   :: Double
                               , nuclideName  :: String
                               }



-- | Atomic number, charge and mass excess (12C scale) of each nuclide:
--
nuclideList = [
        NuclideInfo { atomicNumber =  1, charge = 0, massExcess =  0.008665, nuclideName = "n"   },    -- neutron
        NuclideInfo { atomicNumber =  1, charge = 1, massExcess =  0.007825, nuclideName = "p"   },    -- proton
        NuclideInfo { atomicNumber =  2, charge = 1, massExcess =  0.014102, nuclideName = "2H"  },    -- deuterium
        NuclideInfo { atomicNumber =  3, charge = 1, massExcess =  0.016050, nuclideName = "3H"  },    -- tritium
        NuclideInfo { atomicNumber =  3, charge = 2, massExcess =  0.016030, nuclideName = "3He" },    -- 3He
        NuclideInfo { atomicNumber =  4, charge = 2, massExcess =  0.002603, nuclideName = "4He" },    -- 4He
        NuclideInfo { atomicNumber =  6, charge = 3, massExcess =  0.015125, nuclideName = "6Li" },    -- 6Li
        NuclideInfo { atomicNumber =  7, charge = 3, massExcess =  0.016004, nuclideName = "7Li" },    -- 7Li
        NuclideInfo { atomicNumber =  7, charge = 4, massExcess =  0.016929, nuclideName = "7Be" },    -- 7Be
        NuclideInfo { atomicNumber =  8, charge = 3, massExcess =  0.022487, nuclideName = "8Li" },    -- 8Li
        NuclideInfo { atomicNumber =  8, charge = 5, massExcess =  0.024609, nuclideName = "8B"  },    -- 8B
        NuclideInfo { atomicNumber =  9, charge = 5, massExcess =  0.012186, nuclideName = "9Be" },    -- 9Be
        NuclideInfo { atomicNumber = 10, charge = 5, massExcess =  0.012939, nuclideName = "10B" },    -- 10B
        NuclideInfo { atomicNumber = 11, charge = 5, massExcess =  0.009305, nuclideName = "11B" },    -- 11B
        NuclideInfo { atomicNumber = 11, charge = 6, massExcess =  0.011432, nuclideName = "11C" },    -- 11C
        NuclideInfo { atomicNumber = 12, charge = 5, massExcess =  0.014354, nuclideName = "12B" },    -- 12B
        NuclideInfo { atomicNumber = 12, charge = 6, massExcess =  0.000000, nuclideName = "12C" },    -- 12C
        NuclideInfo { atomicNumber = 12, charge = 7, massExcess =  0.018641, nuclideName = "12N" },    -- 12N
        NuclideInfo { atomicNumber = 13, charge = 6, massExcess =  0.003354, nuclideName = "13C" },    -- 13C
        NuclideInfo { atomicNumber = 13, charge = 7, massExcess =  0.005738, nuclideName = "13N" },    -- 13N
        NuclideInfo { atomicNumber = 14, charge = 6, massExcess =  0.003242, nuclideName = "14C" },    -- 14C
        NuclideInfo { atomicNumber = 14, charge = 7, massExcess =  0.003074, nuclideName = "14N" },    -- 14N
        NuclideInfo { atomicNumber = 14, charge = 8, massExcess =  0.008597, nuclideName = "14O" },    -- 14O
        NuclideInfo { atomicNumber = 15, charge = 7, massExcess =  0.000108, nuclideName = "15N" },    -- 15N
        NuclideInfo { atomicNumber = 15, charge = 8, massExcess =  0.003070, nuclideName = "15O" },    -- 15O
        NuclideInfo { atomicNumber = 16, charge = 8, massExcess = -0.005085, nuclideName = "16O" } ]   -- 16O



-- | projections from the nuclide table:
--
ams :: Vector Double
ams = fromList $ take nNuclides $ map (fromIntegral . atomicNumber) nuclideList

zs  :: Vector Double
zs  = fromList $ take nNuclides $ map (fromIntegral . charge) nuclideList

dms :: Vector Double
dms = fromList $ take nNuclides $ map massExcess nuclideList


-- | helper to compute the first few factorials quickly.  This
--   improves performance by a small but measurable amount.
--
factorial :: Int -> Int
factorial 0 = 1
factorial 1 = 1
factorial 2 = 2
factorial 3 = 6
factorial n = product [1 .. n]


-- | The linearizedReactionMatrix returns the rate at which
--   nuclide i is converted to nuclide j (positive numbers indicate creation,
--   negative, destruction).
--
--   This is a bit of an ugly hack, but using an intermediate list containing the
--   nonzero reaction rates caused many cache unfriendly memory accesses, increasing
--   the running time by about 40 percent.  Here, I generate an unboxed tuple
--   with the reaction rates for reaction number n and copy the nonzero values directly
--   into the linearized reaction rate matrix.
--
linearizedReactionMatrix :: Double          -- ^ temperature in units of 10^9 K
                         -> Double          -- ^ baryon density in units of g cm^{-3}
                         -> Vector Double   -- ^ vector of element abundances
                         -> Matrix Double
linearizedReactionMatrix t9 rhob ys =
        let
                reactionFunc n = let
                
                        reac  = reactionTable ! n

                        typ   = reactionType reac
                        fwd   = forwardFunc reac
                        rev   = reverseFunc reac
                        coeff = revCoeff reac
                        q     = qValue   reac

                        i = nuclideA reac
                        j = nuclideB reac
                        k = nuclideC reac
                        l = nuclideD reac

                        (nA, nB, nC, nD) = reactionConfig typ
                                           
                        yA = if i >= 1 && i <= nNuclides
                             then ys @> (i - 1)
                             else 1.0

                        yB = if j >= 1 && j <= nNuclides
                             then ys @> (j - 1)
                             else 1.0

                        yC = if k >= 1 && k <= nNuclides
                             then ys @> (k - 1)
                             else 1.0

                        yD = if l >= 1 && l <= nNuclides
                             then ys @> (l - 1)
                             else 1.0

                        cA = if nA == 0
                             then 0.0
                             else ((fromIntegral nA) * yA^(nA - 1) * yB^nB) /
                                  fromIntegral ((factorial nA) * (factorial nB) * (nA + nB))

                        cB = if nB == 0
                             then 0.0
                             else ((fromIntegral nB) * yA^nA * yB^(nB - 1)) /
                                  fromIntegral ((factorial nA) * (factorial nB) * (nA + nB))
 
                        cC = if nC == 0
                             then 0.0
                             else ((fromIntegral nC) * yC^(nC - 1) * yD^nD) /
                                  fromIntegral ((factorial nC) * (factorial nD) * (nC + nD))

                        cD = if nD == 0
                             then 0.0
                             else ((fromIntegral nD) * yC^nC * yD^(nD - 1)) /
                                  fromIntegral ((factorial nC) * (factorial nD) * (nC + nD))

                        f  = forwardReaction typ t9 rhob fwd
                        r  = reverseReaction typ t9 rhob fwd rev q coeff

                  in
                    (# i, i, -f * (fromIntegral nA) * cA,
                       j, i, -f * (fromIntegral nB) * cA,     
                       k, i,  f * (fromIntegral nC) * cA,
                       l, i,  f * (fromIntegral nD) * cA,
                       i, j, -f * (fromIntegral nA) * cB,
                       j, j, -f * (fromIntegral nB) * cB,
                       k, j,  f * (fromIntegral nC) * cB,
                       l, j,  f * (fromIntegral nD) * cB,
                       i, k,  r * (fromIntegral nA) * cC,
                       j, k,  r * (fromIntegral nB) * cC,
                       k, k, -r * (fromIntegral nC) * cC,
                       l, k, -r * (fromIntegral nD) * cC,
                       i, l,  r * (fromIntegral nA) * cD,
                       j, l,  r * (fromIntegral nB) * cD,
                       k, l, -r * (fromIntegral nC) * cD,
                       l, l, -r * (fromIntegral nD) * cD #)

          in
            runSTMatrix $ do
              m <- newMatrix 0.0 nNuclides nNuclides
              let
                      update i j v = if i >= 1 && i <= nNuclides && j >= 1 && j <= nNuclides
                                     then do
                                       let
                                               i' = i - 1
                                               j' = j - 1

                                       v' <- unsafeReadMatrix  m i' j'
                                       m' <- unsafeWriteMatrix m i' j' (v + v')
                                       return m'
                                     else do
                                       return ()

                      accum n = case reactionFunc n of 
                                  { (# i0,  j0,  f0,
                                       i1,  j1,  f1,
                                       i2,  j2,  f2,
                                       i3,  j3,  f3,
                                       i4,  j4,  f4,
                                       i5,  j5,  f5,
                                       i6,  j6,  f6,
                                       i7,  j7,  f7,
                                       i8,  j8,  f8,
                                       i9,  j9,  f9,
                                       i10, j10, f10,
                                       i11, j11, f11,
                                       i12, j12, f12,
                                       i13, j13, f13,
                                       i14, j14, f14,
                                       i15, j15, f15 #) ->
                                    do
                                      update i0  j0  f0
                                      update i1  j1  f1
                                      update i2  j2  f2
                                      update i3  j3  f3
                                      update i4  j4  f4
                                      update i5  j5  f5
                                      update i6  j6  f6
                                      update i7  j7  f7
                                      update i8  j8  f8
                                      update i9  j9  f9
                                      update i10 j10 f10
                                      update i11 j11 f11
                                      update i12 j12 f12
                                      update i13 j13 f13
                                      update i14 j14 f14
                                      update i15 j15 f15 }

              mapM_ accum reactionNumberList
              return m


forwardReaction :: ReactionType    -- ^ type of the reaction
                -> Double          -- ^ temperature in units of 10^9 K
                -> Double          -- ^ baryon density in units of g cm^{-3}
                -> Reaction        -- ^ the forward reaction function
                -> Double
forwardReaction typ t9 rhob fwd = 
        case typ of
                WeakDecay   -> fwd t9
                NNgN        -> rhob * fwd t9
                NNNN        -> rhob * fwd t9
                WeakFission -> fwd t9
                NN2N        -> rhob * fwd t9
                N2NN        -> rhob * fwd t9
                N3gN        -> rhob * rhob * fwd t9
                N2NgN       -> rhob * rhob * fwd t9
                NNN2N       -> rhob * fwd t9
                NN3N        -> rhob * fwd t9
                N2N2N       -> rhob * fwd t9


reverseReaction :: ReactionType    -- ^ type of the reaction
                -> Double          -- ^ temperature in units of 10^9 K
                -> Double          -- ^ baryon density in units of g cm^{-3}
                -> Reaction        -- ^ the forward reaction function
                -> Maybe Reaction  -- ^ the reverse reaction function
                -> Double          -- ^ energy release in the forward reaction
                -> Double          -- ^ phase space coefficient for the reverse reaction
                -> Double
reverseReaction typ t9 rhob fwd rev q coeff =
        case rev of
          Just rf -> rf t9
          Nothing -> let
                         f     = fwd t9
                         t932  = t9**1.5
                         t9m32 = 1.0 / t932
                     in case typ of
                           WeakDecay   -> 0.0
                           NNgN        -> coeff * 1.0e+10 * t932 * exp(-q / t9) * f
                           NNNN        -> coeff * exp(-q / t9) * rhob * f
                           WeakFission -> 0.0
                           NN2N        -> coeff * exp(-q / t9) * rhob * f
                           N2NN        -> coeff * exp(-q / t9) * rhob * f
                           N3gN        -> coeff * 1.0e+20 * t932 * t932 * exp(-q / t9) * f
                           N2NgN       -> coeff * 1.0e+20 * t932 * t932 * exp(-q / t9) * f
                           NNN2N       -> coeff * 1.0e-10 * t9m32 * rhob * rhob * exp(-q / t9) * f
                           NN3N        -> coeff * 1.0e-10 * t9m32 * rhob * rhob * exp(-q / t9) * f
                           N2N2N       -> coeff * 1.0e-10 * t9m32 * rhob * rhob * exp(-q / t9) * f

