--
-- Thermal.hs
--


module Thermal where


import Numeric.GSL.Special.Bessel
import Debug.Trace

import Initialize
import Utils



-- | Photon energy density in units of g cm^{-3}
--
photonEnergyDensity :: Double        -- ^ temperature (10^9 K)
                    -> Double
photonEnergyDensity t9 = 8.418 * (t9^4)


-- | Derivative of the photon energy density with respect to t9, in unit
--   of g cm^{-3} (10^9 K)^{-1}.
--
dTempPhotonEnergyDensity :: Double       -- ^ temperature (10^9 K)
                         -> Double
dTempPhotonEnergyDensity t9 = 4.0 * (photonEnergyDensity t9) / t9


-- | Photon pressure / c^2, units gm cm^{-3}
--
photonPressure :: Double             -- ^ temperature (10^9 K)
               -> Double
photonPressure t9 = (photonEnergyDensity t9) / 3.0


-- | Electron plus positron energy density, units g cm^{-3}
--
epEnergyDensity :: Double            -- ^ temperature (10^9 K)
                -> Double            -- ^ electron chemical potential
                -> Double
epEnergyDensity t9 phi =
        let
                z = t9electron / t9  -- ^ z = m_e * c^2 / (k * T)
                f z n = (-1)^(n + 1) * cosh ((fromIntegral n) * phi) * bigM ((fromIntegral n) * z) "epEnergyDensity"
                g z n = (-1)^(n + 1) * exp ((fromIntegral n) * (phi - z)) / ((2.0 * (fromIntegral n) * z)**1.5)
        in
                charDensity * sumSeries f 1 5 z


-- | Derivative of electron plus positron energy density w.r.t. t9,
--   units g cm^{-3} (10^9 K)^{-1}
--
dTempEpEnergyDensity :: Double       -- ^ temperature (10^9 K)
                     -> Double       -- ^ electron chemical potential
                     -> Double
dTempEpEnergyDensity t9 phi =
        let
                z = t9electron / t9  -- z = m_e * c^2 / (k * T)
                f z n = (-1)^(n + 1) * (fromIntegral n) * cosh ((fromIntegral n) * phi) * bigN ((fromIntegral n) * z)
                g z n = (-1)^(n + 1) * (fromIntegral n) * exp ((fromIntegral n) * (phi - z)) / ((2.0 * (fromIntegral n) * z)**1.5)
        in
                charDensity * (z / t9) * sumSeries f 1 5 z


-- | Derivative of electron plus positron energy density w.r.t. phie,
--   units g cm^{-3}.  (The electron chemical potential, phie, is the
--   ratio of the chemical potential energy to kT, so it is
--   dimensionless.)
--
dphiEpEnergyDensity :: Double        -- ^ temperature (10^9 K)
                    -> Double        -- ^ electron chemical potential
                    -> Double
dphiEpEnergyDensity t9 phi =
        let
                z = t9electron / t9  -- z = m_e * c^2 / (k * T)
                f z n = (-1)^(n + 1) * sinh ((fromIntegral n) * phi) * bigM ((fromIntegral n) * z) "dphiEpEnergyDensity"
                g z n = (-1)^(n + 1) * exp ((fromIntegral n) * (phi - z)) / ((2.0 * (fromIntegral n) * z)**1.5)
        in
                 charDensity * sumSeries f 1 5 z



-- | Electron plus positron pressure / c^2, units g cm^{-3}
--
epPressure :: Double                 -- ^ temperature (10^9 K)
           -> Double                 -- ^ electron chemical potential
           -> Double
epPressure t9 phi =
        let
                z = t9electron / t9  -- z = m_e * c^2 / (k * T)
                f z n = ((-1)^(n + 1) / ((fromIntegral n) * z)) * cosh ((fromIntegral n) * phi) * bigL ((fromIntegral n) * z)
                g z n = ((-1)^(n + 1) / ((fromIntegral n) * z)) * exp ((fromIntegral n) * (phi - z)) / ((2.0 * (fromIntegral n) * z)**1.5)
        in
                charDensity * sumSeries f 1 5 z


-- | Electron minus positron number density
--   times (pi^2 / 2) * (hbar / (m_e * c))^3 * z^3, dimensionless.
--
epDiffNumberDensity :: Double        -- ^ temperature (10^9 K)
                    -> Double        -- ^ electron chemical potential
                    -> Double
epDiffNumberDensity t9 phi =
        let
                z = t9electron / t9  -- z = m_e * c^2 / (k * T)
                f z n = (-1)^(n + 1) * sinh ((fromIntegral n) * phi) * bigL ((fromIntegral n) * z)
                g z n = (-1)^(n + 1) * exp ((fromIntegral n) * (phi - z)) / ((2.0 * (fromIntegral n) * z)**1.5)
        in
                (z^3) * sumSeries f 1 5 z


-- | Derivative of the electron minus positron number density with respect to t9,
--   times (pi^2 / 2) * (hbar / (m_e * c))^3 * z^3, dimensionless.
--
dTempEpDiffNumberDensity :: Double   -- ^ temperature (10^9 K)
                         -> Double   -- ^ electron chemical potential
                         -> Double
dTempEpDiffNumberDensity t9 phi =
        let
                z = t9electron / t9  -- z = m_e * c^2 / (k * T)
                f z n = (-1)^(n + 1) * sinh ((fromIntegral n) * phi) * (3.0 * bigL ((fromIntegral n) * z) - z * (fromIntegral n) * bigM ((fromIntegral n) * z) "dTempEpDiffNumberDensity")
                g z n = (-1)^(n + 1) * (fromIntegral n) * exp ((fromIntegral n) * (phi - z)) / ((2.0 * (fromIntegral n) * z)**1.5)
        in
                -((z^3) / t9) * sumSeries f 1 5 z


-- | Derivative of the electron minus positron number density with respect to phi,
--   times (pi^2 / 2) * (hbar / (m_e * c))^3 * z^3, dimensionless.
--
dphiEpDiffNumberDensity :: Double    -- ^ temperature (10^9 K)
                        -> Double    -- ^ electron chemical potential
                        -> Double
dphiEpDiffNumberDensity t9 phi =
        let
                z = t9electron / t9  -- z = m_e * c^2 / (k * T)
                f z n = (-1)^(n + 1) * (fromIntegral n) * cosh ((fromIntegral n) * phi) * bigL ((fromIntegral n) * z)
                g z n = (-1)^(n + 1) * (fromIntegral n) * exp ((fromIntegral n) * (phi - z)) / ((2.0 * (fromIntegral n) * z)**1.5)
        in
                (z^3) * sumSeries f 1 5 z


-- | Electron neutrino energy density, units g cm^{-3}
--
electronNeutrinoEnergyDensity :: Double          -- ^ temperature (10^9 K)
                              -> Double
electronNeutrinoEnergyDensity t9 = 7.366 * (t9^4)


-- | Total neutrino energy density (nondegenerate), units g cm^{-3}
--
neutrinoEnergyDensity :: Double     -- ^ temperature (10^9 K)
                      -> Double     -- ^ normalized density
                      -> Double
neutrinoEnergyDensity t9 hv =
        let
                nNu               = fromIntegral neutrinoSpecies
                rhoNuElectronInit = electronNeutrinoEnergyDensity t9Initial
                rhoBaryon         = baryonEnergyDensity t9 hv
                rhoBaryonInit     = baryonEnergyDensity t9Initial hvInitial
        in
                nNu * rhoNuElectronInit * ((rhoBaryon / rhoBaryonInit)**(4.0 / 3.0))


-- | Baryon energy density, units g cm^{-3}
--
baryonEnergyDensity :: Double       -- ^ temperature (10^9 K)
                    -> Double       -- ^ normalized density
                    -> Double
baryonEnergyDensity t9 hv = hv * (t9^3)


-- | Total energy density, units g cm^{-3}
--
totalEnergyDensity :: Double        -- ^ temperature (10^9 K)
                   -> Double        -- ^ normalized density
                   -> Double        -- ^ normalized electron chemical potential
                   -> Double
totalEnergyDensity t9 hv phi = if t9 > t9small
                                   then (photonEnergyDensity   t9) +
                                        (epEnergyDensity       t9 phi) +
                                        (neutrinoEnergyDensity t9 hv) +
                                        (baryonEnergyDensity   t9 hv)
                                   else (photonEnergyDensity   t9) +
                                        (neutrinoEnergyDensity t9 hv) +
                                        (baryonEnergyDensity   t9 hv)
